"""
Module realize events mixin
"""

from typing import Awaitable, List, Literal, Optional, Union

from luna3.common.http_objs import StatQuery
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class EventsMixin(MixinStub):
    """Events Mixin to work with events."""

    def getEvents(
        self,
        target: Optional[List[str]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        eventIds: Optional[List[str]] = None,
        externalIds: Optional[List[str]] = None,
        handlerIds: Optional[List[str]] = None,
        topSimilarFaceList: Optional[List[str]] = None,
        topSimilarFaceIds: Optional[List[str]] = None,
        topSimilarFaceSimilarityGte: Optional[str] = None,
        topSimilarFaceSimilarityLt: Optional[str] = None,
        ageLt: Optional[int] = None,
        ageGte: Optional[int] = None,
        gender: Optional[Union[int, Literal["null"]]] = None,
        emotions: Optional[List[Union[int, Literal["null"]]]] = None,
        ethnicGroups: Optional[Union[List[int], Literal["null"]]] = None,
        faceIds: Optional[List[str]] = None,
        userData: Optional[str] = None,
        page: Optional[int] = 1,
        pageSize: Optional[int] = 10,
        order: Optional[str] = "desc",
        sources: Optional[List[Union[str, Literal["null"]]]] = None,
        tags: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get events by filters.

        Args:
            target: list of events' fields to receive instead of full events,
                fields should be from ("create_time", "event_id", "descriptor_id",
                "attribute_id", "source", "top_similar_face_id", "top_similar_face_list",
                "top_similar_face_sim", "match_result", "face_id", "attach_result", "gender", "age", "emotions",
                "ethnic_group", "tags", "user_data", "sample_ids")

            createTimeGte: lower including create time boundary
            createTimeLt: upper excluding create time boundary
            faceIds: ids of faces linked with an event
            eventIds: event ids
            handlerIds: handler ids
            externalIds: external ids
            userData: event user data
            sources: list of source
            tags: users tags
            topSimilarFaceList: searched Luna API list
            topSimilarFaceIds: top similar face ids
            topSimilarFaceSimilarityGte: lower including top similarity boundary
            topSimilarFaceSimilarityLt: upper excluding top similarity boundary
            ageLt: upper excluding age boundary
            ageGte: lower including age boundary
            gender: gender of event (0 or 1)
            emotions: dominant emotions in event (one of (1, 2, 3, 4, 5, 6, 7) -
                      matches "anger", "disgust", "fear", "happiness", "neutral", "sadness", "surprise")
            ethnicGroups: dominant ethnicity of the face ((1, 2, 3, 4) -
                                  means (indian, asian, african_american, caucasian)
            page: number of result page
            pageSize: count of results in page
            order: result sort order (ask or desc)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getEvents(gender=1, page=1)
        >>> print(response.statusCode)
        200
        >>> print(response.json)
        {
            'total': 1,
            'events':
                [
                    {
                        'id': 3301,
                        'account_id': 'cf579c23-a9a7-4fe7-abe8-f7a64e088cac',
                        'create_time': '2018-09-26T14:33:12.651854Z',
                        'event_id': 'e325a65d-c6f6-4740-bb2b-04fd5c59cfdb',
                        'descriptor_id': 'b1a89b52-1815-4f17-a867-5fca5f44c6ff',
                        'handler_id': 'e25b6397-a558-4983-bd39-2a992be14405',
                        'attribute_id': 'a5412a46-3936-44f0-b5cc-952fc59b6018',
                        'source': 'moscow',
                        'top_similar_face_id': '7d930bf7-6b68-482f-bbbe-226774f2a0b9',
                        'top_similar_face_list': '5c38b4f2-da69-4cf3-a29b-c8e5f843edde',
                        'top_similar_face_sim': 0.5245573159,
                        'match_result': {'some': {'match': 'result'}},
                        'extract_result': {'some': {'extract': 'result'}},
                        'face_id': '014e5871-2405-42b0-9da2-d7fb9500e8bd',
                        'attach_result': {'some': {'attach': 'result'}},
                        'gender': 1,
                        'age': 87,
                        'emotion': 4,
                        'ethnic_group': 3,
                        'tags': ['tag81', 'tag53', 'tag93', 'tag80'],
                        'user_data': '041347b5-b3bf-443f-9d26-4f2645725266',
                        'sample_ids': [
                            'b3393124-228f-4dee-9803-6bb6827c8636',
                            '9be18816-1b80-43c5-9861-263821c80b4b',
                            'f95bea3e-5a3e-4f1e-a145-8db20a282893'
                        ]
                    }
                ]
        }

        """
        queries = {
            "target": target,
            "create_time__gte": createTimeGte,
            "event_ids": eventIds,
            "external_ids": externalIds,
            "top_similar_face_list": topSimilarFaceList,
            "top_similar_face_ids": topSimilarFaceIds,
            "top_similar_face_similarity__gte": topSimilarFaceSimilarityGte,
            "top_similar_face_similarity__lt": topSimilarFaceSimilarityLt,
            "age__lt": ageLt,
            "age__gte": ageGte,
            "gender": gender,
            "emotions": emotions,
            "ethnic_groups": ethnicGroups,
            "handler_ids": handlerIds,
            "user_data": userData,
            "create_time__lt": createTimeLt,
            "tags": tags,
            "face_ids": faceIds,
            "sources": sources,
            "page": page,
            "page_size": pageSize,
            "order": order,
        }
        return self.makeRequest(f"{self.baseUri}/events", "GET", queryParams=queries, **kwargs)

    def checkEvent(self, eventId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check event with id=eventId existence.

        Args:
            eventId: event id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.checkEvent(eventId='e325a65d-c6f6-4740-bb2b-04fd5c59cfdb')
        >>> print(response.statusCode)
        200

        """
        return self.makeRequest(f"{self.baseUri}/events/{eventId}", "HEAD", **kwargs)

    def getEvent(self, eventId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get event by id.

        Args:
            eventId: event id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getEvent(eventId='e325a65d-c6f6-4740-bb2b-04fd5c59cfdb')
        >>> print(response.statusCode)
        200
        >>> print(response.json)
        {
            'id': 3302,
            'create_time': '2018-09-26T14:33:10.651987Z',
            'event_id': 'e325a65d-c6f6-4740-bb2b-04fd5c59cfdb',
            'descriptor_id': '6c22ee22-4789-4e82-b8d8-fd2b29e46638',
            'handler_id': 'b2858cfd-0d69-410a-ae2c-384f9d1d1082',
            'attribute_id': '70acbc34-685c-4c4e-81ae-2c02e397849d',
            'source': 'moscow',
            'top_similar_face_id': '8590266b-3bd7-432e-bb01-9160abe0e021',
            'top_similar_face_list': '55bb77f0-ec86-498e-b3db-05ce012d3c20',
            'top_similar_face_sim': 0.48428434280000004,
            'match_result': {'some': {'match': 'result'}},
            'extract_result': {'some': {'extract': 'result'}},
            'face_id': '5ba0989d-292b-437b-9805-50277c71147f',
            'attach_result': {'some': {'attach': 'result'}},
            'gender': 0,
            'age': 10,
            'emotion': 4,
            'ethnic_group': 3,
            'tags': ['tag78', 'tag97', 'tag35'],
            'user_data': '041347b5-b3bf-443f-9d26-4f2645725266',
            'sample_ids': ['1b17320a-9799-469d-81a2-ebcccd869364']
        }

        """
        return self.makeRequest(f"{self.baseUri}/events/{eventId}", "GET", **kwargs)

    def getEventsStats(
        self, query: Union[StatQuery, dict] = StatQuery(), **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get events statistics by aggregation (<aggregator> and <groupBy> on <target>) and filters.

        Args:
            query: query object or query dict, contained in StatQuery object

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> query = StatQuery().addTarget('gender', 'count').addGroupBy('dayOfWeek')
        >>> response = client.getEventsStats(query=query)
        >>> print(response.statusCode)
        201
        >>> print(response.json)
        {
            'fields': [
                'Group by time: dayOfWeek',
                "event_id, aggregator 'count'"
            ],
            'stats': [
                ['Sunday', 118],
                ['Monday', 396],
                ['Tuesday', 118],
                ['Wednesday', 2311],
                ['Thursday', 119],
                ['Friday', 117],
                ['Saturday', 120]
            ]
        }

        """
        if not isinstance(query, dict):
            query = query.query
        payload = RequestPayload.buildJson(body=query)
        return self.makeRequest(f"{self.baseUri}/events/statistic", "POST", body=payload, **kwargs)
