"""
Module realize samples mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common import functions
from luna3.common.http_objs import BinaryImage
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class SamplesMixin(MixinStub):
    """Samples Mixin to work with samples."""

    def detectFaces(
        self,
        images: Union[BinaryImage, List[BinaryImage]] = None,
        urls: Optional[List[str]] = None,
        samples: Optional[List[str]] = None,
        estimateHeadPose: int = 0,
        detectLandmarks68: int = 0,
        estimateQuality: int = 0,
        estimateGaze: int = 0,
        estimateEyesAttributes: int = 0,
        estimateEmotions: int = 0,
        estimateMouthAttributes: int = 0,
        pitchThreshold: Optional[int] = None,
        rollThreshold: Optional[int] = None,
        yawThreshold: Optional[int] = None,
        warpedImage: int = 0,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Detect faces on input images.

        Args:
            images: Single image or images list for extraction (supported formats "jpeg, png, tif, ppm, bmp").
            urls: list of urls with images
            samples: list of images' ids
            estimateHeadPose: Available values : 0, 1
            detectLandmarks68: Available values : 0, 1
            estimateQuality: Available values : 0, 1
            estimateGaze: Available values : 0, 1
            estimateEyesAttributes: Available values : 0, 1
            estimateMouthAttributes: Available values : 0, 1
            estimateEmotions: Available values : 0, 1
            pitchThreshold: maximum deviation pitch angle from 0
            rollThreshold: maximum deviation roll angle from 0
            yawThreshold: maximum deviation yaw angle from 0
            warpedImage: Whether input image is a warped or arbitrary image.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with samples, exif, filename for succeeded images and
            errors for failed images will be returned.

        Raises:
            ValueError: if no images were found

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> images = [BinaryImage("/home/user231/Pictures/without_face.png"),
        >>>           BinaryImage("/home/user231/Pictures/test_face.jpg")]
        >>> client = LunaHttpClient()
        >>> response = client.detectFaces(images)
        >>> print(response.statusCode)
        201
        >>> print(response.json)
        {
          "failed_images": [],
          "succeeded_images": [
            {
              "samples": [],
              "filename": "home/user231/Pictures/without_face.png"
            },
            {
              "samples": [
                {
                  "sample_id": "995da415-0b45-4afa-bddd-c9a2be42faa7",
                  "url": "/1/samples/995da415-0b45-4afa-bddd-c9a2be42faa7",
                  "detection": {"rect": {"height": 493, "width": 380, "x": 1052, "y": 185},
                  "landmarks5": [[91, 184], [261, 173], [173, 279], [110, 380], [264, 369]]
                  }
                }
              ],
              "filename": "home/user231/Pictures/test_face.jpg"
            }
          ]
        }
        >>> prefix = "https://upload.wikimedia.org/wikipedia/commons/thumb/"
        >>> urls = [
        >>>     f"{prefix}2/25/Lebron_wizards_2017_%28cropped%29.jpg/800px-Lebron_wizards_2017_%28cropped%29.jpg",
        >>>     f"{prefix}e/e1/Trotting_impala_ram%2C_crop.jpg/1024px-Trotting_impala_ram%2C_crop.jpg"
        >>> ]
        >>> client = LunaHttpClient()
        >>> response = client.detectFaces(urls=urls, detectLandmarks68=1, estimateHeadPose=1)
        >>> print(response.statusCode)
        201
        >>> print(response.json)
        {
          "failed_images": [],
          "succeeded_images": [
            {
              "samples": [
                {
                  "sample_id": "dbcb5eb9-1f15-4b2e-b1f8-d1855bc0cd4c",
                  "url": "/1/samples/dbcb5eb9-1f15-4b2e-b1f8-d1855bc0cd4c",
                  "detection": {
                    "rect": {"height": 192, "width": 149, "x": 247, "y": 104},
                    "landmarks5": [[51, 56], [115, 51],[98, 81], [66, 130], [123, 126]],
                    "landmarks68": [
                        [1, 78], [4, 97], [8, 116], [15, 134], [27, 150],
                        [43, 164], [61, 173], [82, 179], [103, 179], [122, 171],
                        [132, 157], [140, 141], [145, 124], [147, 106],
                        [146, 89], [143, 73], [137, 61], [27, 47], [34, 36],
                        [46, 29], [60, 28], [73, 31], [99, 29], [108, 26],
                        [117, 25], [125, 29], [129, 37], [90, 56], [92, 65],
                        [95, 73], [98, 82], [77, 99], [87, 100], [110, 120],
                        [123, 123], [111, 129], [103, 131], [94, 131]],
                    "attributes": {
                      "head_pose": {
                        "pitch": -2.4370028973,
                        "roll": -8.1191987991,
                        "yaw": -15.8652133942
                      }
                    }
                  }
                }
              ],
              "filename": "https://upload.wikimedia.org/wikipedia/commons/thumb/2/25/
                Lebron_wizards_2017_%28cropped%29.jpg/800px-Lebron_wizards_2017_%28cropped%29.jpg"
            },
            {
              "samples": [],
              "filename": "https://upload.wikimedia.org/wikipedia/commons/thumb/e/e1/
                Trotting_impala_ram%2C_crop.jpg/1024px-Trotting_impala_ram%2C_crop.jpg"
            }
          ]
        }

        """
        if images is None and urls is None and samples is None:
            raise ValueError("Cannot find images, or samples, or urls in input args")

        queries = {
            "estimate_head_pose": estimateHeadPose,
            "detect_landmarks68": detectLandmarks68,
            "estimate_quality": estimateQuality,
            "estimate_gaze": estimateGaze,
            "estimate_eyes_attributes": estimateEyesAttributes,
            "estimate_mouth_attributes": estimateMouthAttributes,
            "estimate_emotions": estimateEmotions,
            "pitch_threshold": pitchThreshold,
            "roll_threshold": rollThreshold,
            "yaw_threshold": yawThreshold,
            "warped_image": warpedImage,
        }

        headers = {}
        if images:
            payload = RequestPayload(body=self.getBodyImages(headers, images), headers=headers)
        else:
            payload = RequestPayload.buildJson(
                body={key: value for key, value in (("urls", urls), ("samples", samples)) if value}, headers=headers
            )
        return self.makeRequest(f"{self.baseUri}/detector", "POST", queryParams=queries, body=payload, **kwargs)

    def saveSample(self, image: BinaryImage, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Save sample to the system.

        Args:
            image: Single image object.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with sample_id and url will be returned.

        Raises:
            ValueError: if body has unsupported mimetype

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> image = BinaryImage("/home/user1/img/face05.jpg")
        >>> client = LunaHttpClient()
        >>> response = client.saveSample(image)
        >>> print(response.statusCode)
        201
        >>> print(response.json)
        {
          "sample_id": "b5d6fd45-fcca-453d-ac05-3e594054b813",
          "url": "/1/samples/b5d6fd45-fcca-453d-ac05-3e594054b813"
        }

        """
        allowedTypes = ("image/jpeg", "image/png", "image/bmp", "image/tiff", "image/gif", "image/x-portable-pixmap")
        functions.checkContentType(image, allowedTypes=allowedTypes)

        headers = {"Content-Type": image.mimetype}
        return self.makeRequest(f"{self.baseUri}/samples", "POST", headers=headers, body=image.body, **kwargs)

    def getSample(self, sampleId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get sample by id

        Args:
            sampleId: sample Id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
                    In body of :class: `~.LunaResponse` - image

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient()
        >>> response = client.getSample("58c40012-4fd1-4b4b-9a92-b32184316ff6")
        >>> print(len(response.body))  # image in the body
        17750
        >>> print(response.statusCode)
        200

        """
        return self.makeRequest(f"{self.baseUri}/samples/{sampleId}", "GET", **kwargs)

    def checkSample(self, sampleId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check to exist a sample with id=sample_id

        Args:
            sampleId: sample Id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
                    In body of :class: `~.LunaResponse` -

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.checkSample("58c40012-4fd1-4b4b-9a92-b32184316ff6")
        >>> print(response.text)

        >>> print(response.statusCode)
        200

        """
        return self.makeRequest(f"{self.baseUri}/samples/{sampleId}", "HEAD", **kwargs)

    def deleteSample(self, sampleId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete sample with id=sample_id

        Args:
            sampleId: sample Id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
                    In body of :class: `~.LunaResponse` -

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.deleteSample("58c40012-4fd1-4b4b-9a92-b32184316ff6")
        >>> print(response.text)

        >>> print(response.statusCode)
        204

        """
        return self.makeRequest(f"{self.baseUri}/samples/{sampleId}", "DELETE", **kwargs)
