"""
Module contains all client and session to luna-platform services
"""

from ssl import SSLContext
from typing import Optional, Union

from .accounts.accounts import AccountsApi
from .configurator.configurator import ConfiguratorApi
from .events.events import EventsApi
from .faces.faces import FacesApi
from .handlers.handlers import HandlersApi
from .image_store.image_store import StoreApi
from .licenses.licenses import LicensesApi
from .luna_lambda.luna_lambda import LambdaApi
from .python_matcher.python_matcher import PythonMatcherApi
from .remote_sdk.remote_sdk import LunaRemoteSDKApi
from .sender.wsclient import LunaWSClient
from .sesions import HTTPSession
from .settings import HTTPSettings
from .streams_retranslator.streams_retranslator import StreamsRetranslatorApi
from .tasks.tasks import TasksApi
from .video_agent.video_agent import VideoAgentApi
from .video_manager.video_manager import VideoManagerApi


class Client:
    """
    Client for all services

    Attributes:
        lunaFaces (FacesApi): service luna-faces
        lunaFaceSamplesStore (StoreApi): service luna-image-store with face samples
        lunaBodySamplesStore (StoreApi): service luna-image-store with body samples
        lunaImageOriginStore (StoreApi): service luna-image-store with origin images
        lunaTasksResultsStore (StoreApi): service luna-image-store with task results
        lunaObjectsStore (StoreApi): service luna-image-store with objects (zip with images?)
        lunaEvents (EventsApi): service luna-events
        lunaTasks (TasksApi): service luna-tasks
        lunaLicenses (LicensesApi): service luna-licenses
        lunaConfigurator (ConfiguratorApi): service luna-configurator
        lunaHandlers (HandlersApi): service luna-handlers
        lunaPythonMatcher (PythonMatcherApi): service luna-python-matcher
        lunaSender (LunaWSClient): service luna-sender
        lunaAccounts (AccountsApi): service luna-accounts
        lunaVideoManager (VideoManagerApi): service luna-video-manager
        lunaVideoAgent (VideoAgentApi): service luna-video-agent
        lunaStreamsRetranslator (StreamsRetranslatorApi): service luna-streams-retranslator
    """

    def __init__(
        self,
        lunaFaces: Optional[FacesApi] = None,
        lunaFaceSamplesStore: Optional[StoreApi] = None,
        lunaBodySamplesStore: Optional[StoreApi] = None,
        lunaImageOriginStore: Optional[StoreApi] = None,
        lunaTasksResultsStore: Optional[StoreApi] = None,
        lunaObjectsStore: Optional[StoreApi] = None,
        lunaEvents: Optional[EventsApi] = None,
        lunaTasks: Optional[TasksApi] = None,
        lunaConfigurator: Optional[ConfiguratorApi] = None,
        lunaHandlers: Optional[HandlersApi] = None,
        lunaPythonMatcher: Optional[PythonMatcherApi] = None,
        lunaLicenses: Optional[LicensesApi] = None,
        lunaSender: Optional[LunaWSClient] = None,
        lunaAccounts: Optional[AccountsApi] = None,
        lunaRemoteSDK: Optional[LunaRemoteSDKApi] = None,
        lunaLambda: Optional[LambdaApi] = None,
        lunaVideoManager: Optional[VideoManagerApi] = None,
        lunaVideoAgent: Optional[VideoAgentApi] = None,
        lunaStreamsRetranslator: Optional[StreamsRetranslatorApi] = None,
    ):
        self.lunaFaces = lunaFaces
        self.lunaFaceSamplesStore = lunaFaceSamplesStore
        self.lunaBodySamplesStore = lunaBodySamplesStore
        self.lunaImageOriginStore = lunaImageOriginStore
        self.lunaTasksResultsStore = lunaTasksResultsStore
        self.lunaObjectsStore = lunaObjectsStore
        self.lunaEvents = lunaEvents
        self.lunaTasks = lunaTasks
        self.lunaLicenses = lunaLicenses
        self.lunaConfigurator = lunaConfigurator
        self.lunaHandlers = lunaHandlers
        self.lunaPythonMatcher = lunaPythonMatcher
        self.lunaSender = lunaSender
        self.lunaAccounts = lunaAccounts
        self.lunaRemoteSDK = lunaRemoteSDK
        self.lunaLambda = lunaLambda
        self.lunaVideoManager = lunaVideoManager
        self.lunaVideoAgent = lunaVideoAgent
        self.lunaStreamsRetranslator = lunaStreamsRetranslator


class Luna3Session:
    """
    Luna3 session.

    Attributes:
         facesSession (HTTPSession|None): luna-faces HTTP session
         faceSampleStoreSession (HTTPSession|None): luna-image-store with face samples HTTP session
         bodySampleStoreSession (HTTPSession|None): luna-image-store with body samples HTTP session
         imageOriginStoreSession (HTTPSession|None): luna-image-store with origin images HTTP session
         tasksResultsStoreSession (HTTPSession|None): luna-image-store with task result HTTP session
         objectsStoreSession (HTTPSession|None): luna-image-store with user objects HTTP session
         eventSession (HTTPSession|None): luna-events HTTP session
         tasksSession (HTTPSession|None): luna-tasks HTTP session
         licensesSession: (HTTPSession|None): luna-licenses HTTP session
         configuratorSession (HTTPSession|None): luna-configurator HTTP session
         handlersSession (HTTPSession|None): luna-handlers HTTP session
         pythonMatcherSession (HTTPSession|None): luna-python-matcher HTTP session
         senderSession (HTTPSession|None): sender HTTP session
         accountsSession (HTTPSession|None): accounts HTTP session
         videoManagerSession (HTTPSession|None): video manager HTTP session
         videoAgentSession (HTTPSession|None): video agent HTTP session
         streamsRetranslatorSession (HTTPSession|None): streams retranslator HTTP session
    """

    def __init__(
        self,
        facesSettings: Optional[HTTPSettings] = None,
        faceSampleStoreSettings: Optional[HTTPSettings] = None,
        bodySampleStoreSettings: Optional[HTTPSettings] = None,
        imageOriginStoreSettings: Optional[HTTPSettings] = None,
        eventSettings: Optional[HTTPSettings] = None,
        tasksResultStoreSettings: Optional[HTTPSettings] = None,
        objectsStoreSettings: Optional[HTTPSettings] = None,
        tasksSettings: Optional[HTTPSettings] = None,
        configuratorSettings: Optional[HTTPSettings] = None,
        handlersSettings: Optional[HTTPSettings] = None,
        pythonMatcherSettings: Optional[HTTPSettings] = None,
        licensesSettings: Optional[HTTPSettings] = None,
        senderSettings: Optional[HTTPSettings] = None,
        accountsSettings: Optional[HTTPSettings] = None,
        remoteSDKSettings: Optional[HTTPSettings] = None,
        lambdaSettings: Optional[HTTPSettings] = None,
        videoManagerSettings: Optional[HTTPSettings] = None,
        videoAgentSettings: Optional[HTTPSettings] = None,
        streamsRetranslatorSettings: Optional[HTTPSettings] = None,
        echo: bool = False,
        ssl: Union[SSLContext, bool, None] = None,
    ):
        kwargs = {"echo": echo, "ssl": ssl}
        self.facesSession = facesSettings and HTTPSession(FacesApi, facesSettings, **kwargs)
        self.faceSampleStoreSession = faceSampleStoreSettings and HTTPSession(
            StoreApi, faceSampleStoreSettings, **kwargs
        )
        self.bodySampleStoreSession = bodySampleStoreSettings and HTTPSession(
            StoreApi, bodySampleStoreSettings, **kwargs
        )
        self.imageOriginStoreSession = imageOriginStoreSettings and HTTPSession(
            StoreApi, imageOriginStoreSettings, **kwargs
        )
        self.tasksResultsStoreSession = tasksResultStoreSettings and HTTPSession(
            StoreApi, tasksResultStoreSettings, **kwargs
        )
        self.objectsStoreSession = objectsStoreSettings and HTTPSession(StoreApi, objectsStoreSettings, **kwargs)
        self.eventSession = eventSettings and HTTPSession(EventsApi, eventSettings, **kwargs)
        self.tasksSession = tasksSettings and HTTPSession(TasksApi, tasksSettings, **kwargs)
        self.licensesSession = licensesSettings and HTTPSession(LicensesApi, licensesSettings, **kwargs)
        self.configuratorSession = configuratorSettings and HTTPSession(ConfiguratorApi, configuratorSettings, **kwargs)
        self.handlersSession = handlersSettings and HTTPSession(HandlersApi, handlersSettings, **kwargs)
        self.pythonMatcherSession = pythonMatcherSettings and HTTPSession(
            PythonMatcherApi, pythonMatcherSettings, **kwargs
        )
        self.senderSession = senderSettings and HTTPSession(LunaWSClient, senderSettings, **kwargs)
        self.accountsSession = accountsSettings and HTTPSession(AccountsApi, accountsSettings, **kwargs)
        self.remoteSDKSession = remoteSDKSettings and HTTPSession(LunaRemoteSDKApi, remoteSDKSettings, **kwargs)
        self.lambdaSession = lambdaSettings and HTTPSession(LambdaApi, lambdaSettings, **kwargs)
        self.videoManagerSession = videoManagerSettings and HTTPSession(VideoManagerApi, videoManagerSettings, **kwargs)
        self.videoAgentSession = videoAgentSettings and HTTPSession(VideoAgentApi, videoAgentSettings, **kwargs)
        self.streamsRetranslatorSession = streamsRetranslatorSettings and HTTPSession(
            StreamsRetranslatorApi, streamsRetranslatorSettings, **kwargs
        )

    def getClient(self, requestId: str) -> Client:  # pylint: disable=R0915
        """
        Get common client for request.

        Args:
            requestId: request id

        Returns:
            common client
        """
        lunaFaces = None
        faceSamplesStore = None
        bodySamplesStore = None
        imageOriginStore = None
        tasksResultsStore = None
        objectsStore = None
        lunaEvents = None
        lunaTasks = None
        lunaLicenses = None
        lunaConfigurator = None
        lunaHandlers = None
        lunaPythonMatcher = None
        lunaSender = None
        lunaAccounts = None
        lunaRemoteSDK = None
        lunaLambda = None
        lunaVideoManager = None
        lunaVideoAgent = None
        lunaStreamsRetranslator = None

        if self.facesSession is not None:
            lunaFaces = self.facesSession.getClient(lunaRequestId=requestId)
        if self.faceSampleStoreSession is not None:
            faceSamplesStore = self.faceSampleStoreSession.getClient(lunaRequestId=requestId)
        if self.bodySampleStoreSession is not None:
            bodySamplesStore = self.bodySampleStoreSession.getClient(lunaRequestId=requestId)
        if self.imageOriginStoreSession is not None:
            imageOriginStore = self.imageOriginStoreSession.getClient(lunaRequestId=requestId)
        if self.tasksResultsStoreSession is not None:
            tasksResultsStore = self.tasksResultsStoreSession.getClient(lunaRequestId=requestId)
        if self.objectsStoreSession is not None:
            objectsStore = self.objectsStoreSession.getClient(lunaRequestId=requestId)
        if self.eventSession is not None:
            lunaEvents = self.eventSession.getClient(lunaRequestId=requestId)
        if self.tasksSession is not None:
            lunaTasks = self.tasksSession.getClient(lunaRequestId=requestId)
        if self.licensesSession is not None:
            lunaLicenses = self.licensesSession.getClient(lunaRequestId=requestId)
        if self.configuratorSession is not None:
            lunaConfigurator = self.configuratorSession.getClient(lunaRequestId=requestId)
        if self.handlersSession is not None:
            lunaHandlers = self.handlersSession.getClient(lunaRequestId=requestId)
        if self.pythonMatcherSession is not None:
            lunaPythonMatcher = self.pythonMatcherSession.getClient(lunaRequestId=requestId)
        if self.senderSession is not None:
            lunaSender = self.senderSession.getClient(lunaRequestId=requestId)
        if self.accountsSession is not None:
            lunaAccounts = self.accountsSession.getClient(lunaRequestId=requestId)
        if self.remoteSDKSession is not None:
            lunaRemoteSDK = self.remoteSDKSession.getClient(lunaRequestId=requestId)
        if self.lambdaSession is not None:
            lunaLambda = self.lambdaSession.getClient(lunaRequestId=requestId)
        if self.videoManagerSession is not None:
            lunaVideoManager = self.videoManagerSession.getClient(lunaRequestId=requestId)
        if self.videoAgentSession is not None:
            lunaVideoAgent = self.videoAgentSession.getClient(lunaRequestId=requestId)
        if self.streamsRetranslatorSession is not None:
            lunaStreamsRetranslator = self.streamsRetranslatorSession.getClient(lunaRequestId=requestId)

        return Client(
            lunaFaces=lunaFaces,
            lunaFaceSamplesStore=faceSamplesStore,
            lunaBodySamplesStore=bodySamplesStore,
            lunaImageOriginStore=imageOriginStore,
            lunaTasksResultsStore=tasksResultsStore,
            lunaObjectsStore=objectsStore,
            lunaEvents=lunaEvents,
            lunaTasks=lunaTasks,
            lunaConfigurator=lunaConfigurator,
            lunaHandlers=lunaHandlers,
            lunaPythonMatcher=lunaPythonMatcher,
            lunaLicenses=lunaLicenses,
            lunaSender=lunaSender,
            lunaAccounts=lunaAccounts,
            lunaRemoteSDK=lunaRemoteSDK,
            lunaLambda=lunaLambda,
            lunaVideoManager=lunaVideoManager,
            lunaVideoAgent=lunaVideoAgent,
            lunaStreamsRetranslator=lunaStreamsRetranslator,
        )

    async def close(self):
        """Close all active HTTP sessions"""
        for attrValue in self.__dict__.values():
            if isinstance(attrValue, HTTPSession):
                await attrValue.close()
