"""
Base client for luna-api client
"""

from numbers import Real
from typing import Awaitable, Dict, Iterable, List, Optional, Type, Union

import aiohttp
from aiohttp import BasicAuth

from luna3.common import functions
from luna3.common.base_api import BaseHTTPAPI
from luna3.common.http_objs import BinaryImage
from luna3.common.logger import logger
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload


class BaseLunaHttpClient(BaseHTTPAPI):
    """
    Class for request to luna API.

    Attributes:
        origin: luna-api protocol, host and port; default "http://127.0.0.1:5000"
        api: api version of luna-api, default 0
        asyncRequest: default mode for request async or blocking
        lunaRequestId: Luna-Request-Id.
        totalTimeout (Real): request processing timeout in seconds
        connectTimeout (Real): request socket connect timeout or obtaining connection from session connection pool
        sockReadTimeout (Real): request socket read timeout
        sockConnectTimeout (Real): request socket connect timeout
        accountId (str): account id
    """

    def __init__(
        self,
        accountId: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
        origin: str = "http://127.0.0.1:5000",
        api: int = 6,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        multipartBoundary: str = "LUNACOREB0UnDaRyStRiNg",
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        **kwargs,
    ) -> None:
        super().__init__(
            origin,
            api,
            asyncRequest,
            lunaRequestId,
            totalTimeout,
            connectTimeout,
            sockReadTimeout,
            sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )
        self.multipartBoundary = multipartBoundary

        self._checkAuthArgs(accountId=accountId, login=login, password=password, token=token)
        self.accountId = accountId
        self.token = token
        self.login = login
        self.password = password

    @staticmethod
    def _checkAuthArgs(
        accountId: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
    ):
        """
        Check authorization arguments compatibility
        Args:
            accountId: account id
            login: login
            password: password
            token: token
        """
        if (accountId is not None, login is not None, token is not None).count(True) > 1:
            raise RuntimeError("Only one authorization type can be used at a time")
        if (login is not None, password is not None).count(True) == 1:
            raise RuntimeError("It is required 'login' and 'password' for basic authorization")

    def _connectionChecker(self, reply: LunaResponse) -> bool:
        """
        Check Luna API server version from response

        Args:
            reply: LunaResponse
        Returns:
            status in bool
        """
        if reply.success and int(reply.json["luna-api"]["api"]) == self.api:
            return True
        if not reply.success:
            logger.get(reply.requestId).warning(
                f"Connection check is failed, " f"status code: {reply.statusCode}, error: {reply.json}"
            )
        else:
            logger.get(reply.requestId).warning(
                f"Connection check is failed, service "
                f"version {reply.json['luna-api']['api']} "
                f"does not match with version {self.api} from settings"
            )
        return False

    def getStandardHeaders(
        self,
        lunaRequestId: Optional[str] = None,
        accountId: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
    ) -> Dict[str, str]:
        """
        Get standard headers ("LUNA-Account-Id", "LUNA-Request-Id") for request.
        Args:
            lunaRequestId:  Luna request Id . Pattern:
                ^[0-9]{10},[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}$
            accountId:  account id. Pattern:
                ^[0-9]{10},[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}$
            login: login
            password: password
            token: token
        Returns:
            dict with corresponding headers if values are not None
        """
        headers = self.getRequestIdAsDict(lunaRequestId)
        headers.update(self.getAuthHeader(login=login, password=password, token=token, accountId=accountId))
        return headers

    def getBodyImages(
        self, headers: dict, images: Union[BinaryImage, List[BinaryImage]] = None
    ) -> Union[aiohttp.MultipartWriter, bytes, None]:
        """
        Function prepare body and content type for request, depends on incoming image(s).

        Args:
            headers: request's headers to update
            images: Single image or images list for extraction or None

        Returns:
            list of image(s) and list of url(s)
        """
        if images is None:
            return None
        headers.update(functions.makeContentType(images, self.multipartBoundary))
        allowedTypes = (
            "image/jpeg",
            "image/png",
            "image/bmp",
            "image/x-portable-pixmap",
            "image/tiff",
            "multipart/form-data",
            "application/json",
            "image/x-jpeg-base64",
            "image/x-png-base64",
            "image/x-bmp-base64",
            "image/x-tiff-base64",
            "image/x-portable-pixmap-base64",
        )
        functions.checkContentType(images, allowedTypes)
        bodyImages = self.getBodiesOfImages(images)
        return bodyImages

    def getBodiesOfImages(
        self, images: Union[BinaryImage, Iterable[BinaryImage]]
    ) -> Union[aiohttp.MultipartWriter, bytes]:
        """
        Return aiohttp request body or body_producer.

        Args:
            images: image or list of images
        Returns:
            request body
        """
        if isinstance(images, BinaryImage):
            bodyImages = images.body
        else:
            with aiohttp.MultipartWriter("form-data", self.multipartBoundary) as writer:
                for img in images:
                    writer.append(img.body, img.httpMultipartHeaders)
            bodyImages = writer
        return bodyImages

    def getAuthHeader(
        self,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
        accountId: Optional[str] = None,
    ) -> dict:
        """
        Get authorization header by token or login and password

        Args:
            login: login
            password: password
            token: token
            accountId: account id

        Returns:
            dict with Authorization header if applicable or empty dict
        """
        self._checkAuthArgs(accountId=accountId, login=login, password=password, token=token)

        if not accountId is login is token is None:
            if token is not None:
                return {"Authorization": f"Bearer {token}"}
            if login is not None:
                return {"Authorization": BasicAuth(login=login, password=password).encode()}
            if accountId is not None:
                return {"Luna-Account-Id": accountId}

        if self.token is not None:
            return {"Authorization": f"Bearer {self.token}"}
        if self.login is not None:
            return {"Authorization": BasicAuth(login=self.login, password=self.password).encode()}
        if self.accountId is not None:
            return {"Luna-Account-Id": self.accountId}

        return {}

    def makeRequest(  # pylint: disable=arguments-differ
        self,
        url: str,
        method: str,
        queryParams: Optional[dict] = None,
        body: Optional[Union[str, bytes, dict, list, aiohttp.Payload, RequestPayload]] = None,
        headers: Optional[dict] = None,
        asyncRequest: Optional[bool] = None,
        lunaRequestId: Optional[str] = None,
        accountId: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
        **kwargs,
    ):
        """
        Prepare account header and make request.

        Args:
            url: request URL
            method: request method
            queryParams: query parameters
            body: request body
            headers: headers for request
            asyncRequest: execution in asynchronous mode, disabled by default
            lunaRequestId: External request id. Helps uniquely identifying messages, corresponding to particular
                requests, in system logs. Pattern:
                ^[0-9]{10},[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}$
            accountId: account id
            login: login
            password: password
            token: token

        Returns:
            structure with status code, request and decoded Luna API response body is returned.
        """
        requestHeaders = self.getAuthHeader(accountId=accountId, login=login, password=password, token=token)
        if headers:
            requestHeaders.update(headers)
        return super().makeRequest(
            url=url,
            method=method,
            queryParams=queryParams,
            body=body,
            headers=requestHeaders,
            lunaRequestId=lunaRequestId,
            asyncRequest=asyncRequest,
            **kwargs,
        )

    def getConfig(
        self,
        acceptHeaders: Optional[dict] = None,
        lunaRequestId: Optional[str] = None,
        asyncRequest: Optional[bool] = None,
        raiseError: Optional[bool] = False,
        totalTimeout: Optional[Real] = None,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get config of luna service

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getConfig

        Args:
            acceptHeaders: Accept request header. By default Accept-Type is application/json.
            lunaRequestId: External request id. Helps uniquely identifying messages, corresponding to particular
                requests, in system logs. Pattern:
                ^[0-9]{10},[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}$
            asyncRequest: execution in asynchronous mode, disabled by default
            raiseError: if request fails, LunaApiException is raised
            totalTimeout: request processing timeout in seconds.
            connectTimeout: connection timeout seconds.
            sockReadTimeout: socket read timeout
            sockConnectTimeout: socket connect timeout

        Returns:
            LunaResponse: body contains json with HTML or YAML as bytes.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> response= client.getConfig()
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json["LUNA_MONITORING"], sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "BUCKET": "luna_monitoring",
            "FLUSHING_PERIOD": 1,
            "HOST": ...,
            "ORGANIZATION": "luna",
            "PORT": 8086,
            "SEND_DATA_FOR_MONITORING": 1,
            "STORAGE_TYPE": "influx",
            "TOKEN": "********",
            "USE_SSL": 0
        }

        """
        standardHeaders = self.getRequestIdAsDict(lunaRequestId)
        standardHeaders.update({"Accept": "application/json"} if acceptHeaders is None else acceptHeaders)

        return self.makeRequest(
            self.baseUri + "/config",
            "GET",
            headers=standardHeaders,
            asyncRequest=self.getAsyncMode(asyncRequest),
            raiseError=raiseError,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
        )

    def getFeatures(
        self,
        lunaRequestId: Optional[str] = None,
        asyncRequest: Optional[bool] = None,
        raiseError: Optional[bool] = False,
        totalTimeout: Optional[Real] = None,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get platform features

        Args:
            lunaRequestId: External request id. Helps uniquely identifying messages, corresponding to particular
                requests, in system logs. Pattern:
                ^[0-9]{10},[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}$
            asyncRequest: execution in asynchronous mode, disabled by default
            raiseError: if request fails, LunaApiException is raised
            totalTimeout: request processing timeout in seconds.
            connectTimeout: connection timeout seconds.
            sockReadTimeout: socket read timeout
            sockConnectTimeout: socket connect timeout

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with platform features.
        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client

        >>> response = client.getFeatures()
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, indent=4))  # doctest: +ELLIPSIS
        {
            "license": ...,
            "estimations": {
                "people_counter": ...,
                "face_quality": ...,
                "body_attributes": ...,
                "liveness": ...,
                "deepfake": ...
            },
            "components": {
                "events": ...,
                "tasks": ...,
                "sender": ...,
                "image_store": ...,
                "handlers": ...
            }
        }
        """
        return self.makeRequest(
            self.baseUri + "/features",
            "GET",
            lunaRequestId=lunaRequestId,
            asyncRequest=asyncRequest,
            raiseError=raiseError,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
        )


class MixinStub:
    """Abstract stub."""

    # Base service uri; real property is defined above (BaseLunaHttpClient)
    baseUri: str
    # multipart boundary; real property is defined above (BaseLunaHttpClient)
    multipartBoundary: str
    # session object; real property is defined above (BaseLunaHttpClient)
    _session: aiohttp.ClientSession
    # standard headers; real method is defined above (BaseLunaHttpClient)
    getStandardHeaders: Type[BaseLunaHttpClient.getStandardHeaders]
    # get auth header: real method is defined above (BaseLunaHttpClient)
    getAuthHeader: Type[BaseLunaHttpClient.getAuthHeader]
    # get body images; real method is defined above (BaseLunaHttpClient)
    getBodyImages: Type[BaseLunaHttpClient.getBodyImages]
    # get async mode; real method is defined above (BaseLunaHttpClient)
    getAsyncMode: Type[BaseLunaHttpClient.getAsyncMode]
    # get async mode; real method is defined above (BaseLunaHttpClient)
    getBodiesOfImages: Type[BaseLunaHttpClient.getBodiesOfImages]
    # make request; real method is defined above (BaseLunaHttpClient)
    makeRequest: Type[BaseLunaHttpClient.makeRequest]
