"""
Module realize events mixin
"""

from typing import Awaitable, List, Optional, Union

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from luna3.common.http_objs import GeneralStatQuery, StatQuery
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.events.http_objs import CLOTHING_COLOR, EVENT_TARGET
from luna3.lunavl.mixins.base import MixinStub


class EventsMixin(MixinStub):
    """Events Mixin to work with events."""

    def getEvents(  # pylint: disable=R0914
        self,
        target: Optional[List[EVENT_TARGET]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        endTimeGte: Optional[str] = None,
        endTimeLt: Optional[str] = None,
        insertTimeGte: Optional[str] = None,
        insertTimeLt: Optional[str] = None,
        eventIds: Optional[List[str]] = None,
        eventIdGte: Optional[str] = None,
        eventIdLt: Optional[str] = None,
        externalIds: Optional[List[str]] = None,
        handlerIds: Optional[List[str]] = None,
        topMatchingCandidatesLabel: Optional[List[str]] = None,
        topSimilarObjectIds: Optional[List[str]] = None,
        topSimilarExternalIds: Optional[List[str]] = None,
        topSimilarObjectSimilarityGte: Optional[float] = None,
        topSimilarObjectSimilarityLt: Optional[float] = None,
        ageLt: Optional[int] = None,
        ageGte: Optional[int] = None,
        gender: Optional[Union[int, Literal["null"]]] = None,
        emotions: Optional[List[Union[int, Literal["null"]]]] = None,
        masks: Optional[List[Union[int, Literal["null"]]]] = None,
        ethnicGroups: Optional[Union[List[int], Literal["null"]]] = None,
        liveness: Optional[Union[List[int], Literal["null"]]] = None,
        deepfake: Optional[Union[List[int], Literal["null"]]] = None,
        apparentAgeGte: Optional[int] = None,
        apparentAgeLt: Optional[int] = None,
        apparentGender: Optional[List[Union[Literal[0, 1, 2], Literal["null"]]]] = None,
        headwearStates: Optional[List[Union[Literal[0, 1, 2], Literal["null"]]]] = None,
        sleeveLengths: Optional[List[Union[Literal["short", "long", "undefined"], Literal["null"]]]] = None,
        upperClothingColors: Optional[List[Union[CLOTHING_COLOR, Literal["null"]]]] = None,
        backpackStates: Optional[List[Union[Literal[0, 1, 2], Literal["null"]]]] = None,
        faceIds: Optional[List[str]] = None,
        userData: Optional[str] = None,
        sources: Optional[List[Union[str, Literal["null"]]]] = None,
        streamIds: Optional[List[Union[str, Literal["null"]]]] = None,
        cities: Optional[List[Union[str, Literal["null"]]]] = None,
        areas: Optional[List[Union[str, Literal["null"]]]] = None,
        districts: Optional[List[Union[str, Literal["null"]]]] = None,
        streets: Optional[List[Union[str, Literal["null"]]]] = None,
        houseNumbers: Optional[List[Union[str, Literal["null"]]]] = None,
        originLongitude: Optional[Union[float, Literal["null"]]] = None,
        originLatitude: Optional[Union[float, Literal["null"]]] = None,
        longitudeDelta: Optional[float] = None,
        latitudeDelta: Optional[float] = None,
        tags: Optional[List[str]] = None,
        trackIds: Optional[List[Union[str, Literal["null"]]]] = None,
        meta: Optional[dict] = None,
        page: Optional[int] = 1,
        pageSize: Optional[int] = 10,
        order: Optional[str] = "desc",
        accountIdFilter: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get events by filters.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getEvents

        Args:
            target: list of events' fields to receive instead of full events
            createTimeGte: lower including create time boundary
            createTimeLt: upper excluding create time boundary
            endTimeGte: lower including end time boundary
            endTimeLt: upper excluding end time boundary
            insertTimeGte: lower including insert time boundary
            insertTimeLt: upper excluding insert time boundary
            faceIds: ids of faces linked with an event
            eventIds: event ids
            eventIdGte: event id lower including bound
            eventIdLt: event id upper excluding bound
            handlerIds: handler ids
            externalIds: external ids
            userData: event user data
            sources: list of source
            streamIds: list of stream ids
            areas: list of areas
            streets: list of streets
            districts: list of districts
            cities: list of cities
            houseNumbers: list of house numbers
            originLongitude: origin longitude in degrees
            originLatitude:  origin latitude in degrees
            longitudeDelta: longitude delta in degrees
            latitudeDelta: latitude delta in degrees
            tags: users tags
            topMatchingCandidatesLabel: top matching candidate label
            topSimilarObjectIds: top similar object ids
            topSimilarExternalIds: top similar external ids
            topSimilarObjectSimilarityGte: lower including top similarity boundary
            topSimilarObjectSimilarityLt: upper excluding top similarity boundary
            ageLt: upper excluding age boundary
            ageGte: lower including age boundary
            gender: gender of event (0 or 1)
            emotions: dominant emotions in event (one of (1, 2, 3, 4, 5, 6, 7) -
                      matches "anger", "disgust", "fear", "happiness", "neutral", "sadness", "surprise")
            masks: dominant masks in event (one of (1, 2, 3)) - matches "missing", "medical mask", "occluded")
            ethnicGroups: dominant ethnicity of the face ((1, 2, 3, 4) -
                                  means (indian, asian, african_american, caucasian)
            trackIds: list of event track ids
            liveness: liveness in event (0 - spoof, 1 - real, 2 - unknown)
            deepfake: deepfake in event (0 - fake, 1 - real)
            apparentAgeLt: apparent age upper excluding boundary
            apparentAgeGte: apparent age upper including boundary
            apparentGender: apparent gender estimation values (0 - female, 1 - male, 2 - undefined)
            headwearStates: headwear states (0 - absent, 1 - present, 2 - undefined)
            sleeveLengths: sleeve lengths
            upperClothingColors: upper clothing color set
            backpackStates: backpack states (0 - absent, 1 - present, 2 - undefined)
            meta: user-defined event meta's filters in format {meta.<path.to.field>__<operator>:<type>: <value>}
            page: number of result page
            pageSize: count of results in page
            order: result sort order (ask or desc)
            accountIdFilter: account id to get events

        Returns:
            LunaResponse: body contains json with info about events.

        >>> import json
        >>> from luna3.common.http_objs import Policies, DetectPolicy
        >>> from testData.resources import FACE_WARP
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> policies = Policies(detectPolicy=DetectPolicy(detectFace=1))
        >>> handlerId = client.createHandler(
        ...     description="test_handler_events", policies=policies, raiseError=True
        ... ).json["handler_id"]
        >>> eventId = client.emitEvents(
        ...     handlerId=handlerId,
        ...     inputData=[FACE_WARP],
        ...     city="Kazan",
        ...     street="Saban",
        ...     houseNumber="5",
        ...     imageType=1,
        ...     useExifInfo=1,
        ...     meta={"foo": "bar"},
        ...     raiseError=True,
        ... ).json["events"][0]["event_id"]
        >>> response = client.getEvents(handlerIds=[handlerId], meta={"meta.foo__in:string": ["bar", "baz"]})
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "events": [
                {
                    "accessories": null,
                    "account_id": ...,
                    "age": null,
                    "attach_result": [],
                    "body_basic_attributes": null,
                    "body_detections": null,
                    "create_time": ...,
                    "deepfake": null,
                    "emotion": null,
                    "end_time": ...,
                    "ethnic_group": null,
                    "event_id": ...,
                    "external_id": "",
                    "face_detections": [
                        {
                            "detect_time": ...,
                            "detect_ts": null,
                            "image_origin": null,
                            "sample_id": ...
                        }
                    ],
                    "face_id": null,
                    "gender": null,
                    "handler_id": ...",
                    "liveness": null,
                    "location": {
                        "area": null,
                        "city": "Kazan",
                        "district": null,
                        "geo_position": null,
                        "house_number": "5",
                        "street": "Saban"
                    },
                    "lower_body": null,
                    "mask": null,
                    "match_result": null,
                    "meta": {
                        "foo": "bar"
                    },
                    "source": null,
                    "stream_id": null,
                    "tags": null,
                    "top_match": null,
                    "track_id": null,
                    "upper_body": null,
                    "user_data": ""
                }
            ]
        }
        >>> print(response.statusCode)
        200

        """
        queries = {
            "target": target,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "end_time__gte": endTimeGte,
            "end_time__lt": endTimeLt,
            "insert_time__gte": insertTimeGte,
            "insert_time__lt": insertTimeLt,
            "event_ids": eventIds,
            "event_id__gte": eventIdGte,
            "event_id__lt": eventIdLt,
            "external_ids": externalIds,
            "top_matching_candidates_label": topMatchingCandidatesLabel,
            "top_similar_object_ids": topSimilarObjectIds,
            "top_similar_external_ids": topSimilarExternalIds,
            "top_similar_object_similarity__gte": topSimilarObjectSimilarityGte,
            "top_similar_object_similarity__lt": topSimilarObjectSimilarityLt,
            "age__lt": ageLt,
            "age__gte": ageGte,
            "gender": gender,
            "emotions": emotions,
            "masks": masks,
            "ethnic_groups": ethnicGroups,
            "liveness": liveness,
            "deepfake": deepfake,
            "apparent_gender": apparentGender,
            "apparent_age__gte": apparentAgeGte,
            "apparent_age__lt": apparentAgeLt,
            "headwear_states": headwearStates,
            "sleeve_lengths": sleeveLengths,
            "upper_clothing_colors": upperClothingColors,
            "backpack_states": backpackStates,
            "handler_ids": handlerIds,
            "user_data": userData,
            "tags": tags,
            "face_ids": faceIds,
            "sources": sources,
            "stream_ids": streamIds,
            "streets": streets,
            "cities": cities,
            "house_numbers": houseNumbers,
            "areas": areas,
            "districts": districts,
            "origin_longitude": originLongitude,
            "origin_latitude": originLatitude,
            "longitude_delta": longitudeDelta,
            "latitude_delta": latitudeDelta,
            "track_ids": trackIds,
            **(meta or {}),
            "page": page,
            "page_size": pageSize,
            "order": order,
            "account_id": accountIdFilter,
        }
        return self.makeRequest(f"{self.baseUri}/events", "GET", queryParams=queries, **kwargs)

    def checkEvent(
        self, eventId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check event with id=eventId existence.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkEvent

        Args:
            eventId: event id
            accountIdFilter: account id to check event

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.common.http_objs import Policies, DetectPolicy
        >>> from testData.resources import FACE_WARP
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> policies = Policies(detectPolicy=DetectPolicy(detectFace=1))
        >>> handlerId = client.createHandler(
        ...     description="test_handler_events", policies=policies, raiseError=True
        ... ).json["handler_id"]
        >>> eventId = client.emitEvents(
        ...     handlerId=handlerId,
        ...     inputData=[FACE_WARP],
        ...     city="Kazan",
        ...     street="Saban",
        ...     houseNumber="5",
        ...     imageType=1,
        ...     useExifInfo=1,
        ...     raiseError=True,
        ... ).json["events"][0]["event_id"]
        >>> response = client.checkEvent(eventId=eventId)
        >>> print(response.statusCode)
        200

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/events/{eventId}", "HEAD", queryParams=queries, **kwargs)

    def getEvent(
        self, eventId: str, target: Optional[List[EVENT_TARGET]] = None, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get event by id.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getEvent

        Args:
            eventId: event id
            target: list of events' fields to receive instead of full events
            accountIdFilter: account id to get event

        Returns:
            LunaResponse: body contains json with event info.

        >>> import json
        >>> from luna3.common.http_objs import Policies, DetectPolicy
        >>> from testData.resources import FACE_WARP
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> policies = Policies(detectPolicy=DetectPolicy(detectFace=1))
        >>> handlerId = client.createHandler(
        ...     description="test_handler_events", policies=policies, raiseError=True
        ... ).json["handler_id"]
        >>> eventId = client.emitEvents(
        ...     handlerId=handlerId,
        ...     inputData=[FACE_WARP],
        ...     city="Kazan",
        ...     street="Saban",
        ...     houseNumber="5",
        ...     imageType=1,
        ...     useExifInfo=1,
        ...     meta={"foo": "bar"},
        ...     raiseError=True,
        ... ).json["events"][0]["event_id"]
        >>> response = client.getEvent(eventId=eventId)
        >>> print(response.statusCode)
        200

        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "accessories": null,
            "account_id": ...,
            "age": null,
            "attach_result": [],
            "body_basic_attributes": null,
            "body_detections": null,
            "create_time": ...,
            "deepfake": null,
            "emotion": null,
            "end_time": ...,
            "ethnic_group": null,
            "event_id": ...,
            "external_id": "",
            "face_detections": [
                {
                    "detect_time": ...,
                    "detect_ts": null,
                    "image_origin": null,
                    "sample_id": ...
                }
            ],
            "face_id": null,
            "gender": null,
            "handler_id": ...,
            "liveness": null,
            "location": {
                "area": null,
                "city": "Kazan",
                "district": null,
                "geo_position": null,
                "house_number": "5",
                "street": "Saban"
            },
            "lower_body": null,
            "mask": null,
            "match_result": null,
            "meta": {
                "foo": "bar"
            },
            "source": null,
            "stream_id": null,
            "tags": null,
            "top_match": null,
            "track_id": null,
            "upper_body": null,
            "user_data": ""
        }
        """
        queries = {"target": target, "account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/events/{eventId}", "GET", queryParams=queries, **kwargs)

    def getEventsStats(
        self, query: Union[StatQuery, dict] = StatQuery(), **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get events statistics by aggregation (<aggregator> and <groupBy> on <target>) and filters.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getEventStats

        Args:
            query: query object or query dict, contained in StatQuery object

        Returns:
            LunaResponse: body contains json with event stats.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.common.http_objs import StatQuery
        >>>
        >>> statQuery = StatQuery().addGroupBy("dayOfWeek").addTarget(column="city")
        >>> response = client.getEventsStats(query=statQuery)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "fields": [
                "Group by time: dayOfWeek",
                "city, aggregator 'count'"
            ],
            "stats": [
                [...]
            ]
        }

        """
        if not isinstance(query, dict):
            query = query.query
        payload = RequestPayload.buildJson(body=query)
        return self.makeRequest(f"{self.baseUri}/events/statistic", "POST", body=payload, **kwargs)

    def getGeneralEvents(  # pylint: disable=R0914
        self,
        eventType: Optional[list[str]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        endTimeGte: Optional[str] = None,
        endTimeLt: Optional[str] = None,
        insertTimeGte: Optional[str] = None,
        insertTimeLt: Optional[str] = None,
        accountIdFilter: Optional[str] = None,
        eventIds: Optional[list[str]] = None,
        eventIdGte: Optional[str] = None,
        eventIdLt: Optional[str] = None,
        sources: Optional[List[Union[str, Literal["null"]]]] = None,
        streamIds: Optional[list[Union[str, Literal["null"]]]] = None,
        trackIds: Optional[list[Union[str, Literal["null"]]]] = None,
        cities: Optional[list[Union[str, Literal["null"]]]] = None,
        areas: Optional[list[Union[str, Literal["null"]]]] = None,
        districts: Optional[list[Union[str, Literal["null"]]]] = None,
        streets: Optional[list[Union[str, Literal["null"]]]] = None,
        houseNumbers: Optional[list[Union[str, Literal["null"]]]] = None,
        originLongitude: Optional[Union[float, Literal["null"]]] = None,
        originLatitude: Optional[Union[float, Literal["null"]]] = None,
        longitudeDelta: Optional[float] = None,
        latitudeDelta: Optional[float] = None,
        eventContentFilters: Optional[dict] = None,
        *,
        page: Optional[int] = 1,
        pageSize: Optional[int] = 10,
        order: Optional[str] = "desc",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get general events by filters.

        Args:
            eventType: general event types
            createTimeGte: lower including create time boundary
            createTimeLt: upper excluding create time boundary
            endTimeGte: lower including end time boundary
            endTimeLt: upper excluding end time boundary
            insertTimeGte: lower including insert time boundary
            insertTimeLt: upper excluding insert time boundary
            accountIdFilter: event account id
            eventIds: event ids
            eventIdGte: event id lower including bound
            eventIdLt: event id upper excluding bound
            sources: list of source
            streamIds: list of stream ids
            trackIds: list of track ids
            cities: list of cities
            areas: list of areas
            districts: list of districts
            streets: list of streets
            houseNumbers: list of house numbers
            originLongitude: origin longitude in degrees
            originLatitude:  origin latitude in degrees
            longitudeDelta: longitude delta in degrees
            latitudeDelta: latitude delta in degrees
            eventContentFilters: event json content filters
            page: number of result page
            pageSize: count of results in page
            order: result sort order (ask or desc)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with events will be returned.

        """
        queries = {
            "event_type": eventType,
            "event_create_time__gte": createTimeGte,
            "event_create_time__lt": createTimeLt,
            "event_end_time__gte": endTimeGte,
            "event_end_time__lt": endTimeLt,
            "insert_time__gte": insertTimeGte,
            "insert_time__lt": insertTimeLt,
            "account_id": accountIdFilter,
            "event_ids": eventIds,
            "event_id__gte": eventIdGte,
            "event_id__lt": eventIdLt,
            "sources": sources,
            "stream_ids": streamIds,
            "track_ids": trackIds,
            "cities": cities,
            "areas": areas,
            "districts": districts,
            "streets": streets,
            "house_numbers": houseNumbers,
            "origin_longitude": originLongitude,
            "origin_latitude": originLatitude,
            "longitude_delta": longitudeDelta,
            "latitude_delta": latitudeDelta,
            "page": page,
            "page_size": pageSize,
            "order": order,
            **(eventContentFilters or {}),
        }
        return self.makeRequest(f"{self.baseUri}/general/events", "GET", queryParams=queries, **kwargs)

    def getGeneralEvent(
        self, eventId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get general event by id.

        Args:
            eventId: event id
            accountIdFilter: event account id filter

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with events will be returned.

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/general/events/{eventId}", "GET", queryParams=queries, **kwargs)

    def getGeneralEventsStats(
        self, query: Union[StatQuery, dict] = GeneralStatQuery(), **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get general events statistics by aggregation (<aggregator> and <groupBy> on <target>) and filters.

        Args:
            query: query object or query dict, contained in GeneralStatQuery object

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with event statistics will be returned.

        """
        if not isinstance(query, dict):
            query = query.query
        payload = RequestPayload.buildJson(body=query)
        return self.makeRequest(f"{self.baseUri}/general/events/statistic", "POST", body=payload, **kwargs)
