"""
Module realizes Video Manager mixin
"""

from contextlib import contextmanager
from numbers import Real
from typing import Awaitable, List, Literal, Optional, Union

import aiohttp

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.common.ws_base import WSBase
from luna3.lunavl.mixins.base import MixinStub
from luna3.public.sdk import VideoPTS
from luna3.video_manager.http_objs import Location, Restart, StreamAnalytic
from luna3.video_manager.utils import getGeneralStreamBody


class VideoManagerMixin(MixinStub, WSBase):
    """
    Video Manager mixin class.
    """

    def createStream(
        self,
        *,
        streamType: Literal["stream", "videofile"],
        reference: str,
        analytics: List[StreamAnalytic],
        downloadable: bool = False,
        timestampSource: Literal["pts", "server", "frame_rate", "auto"] = "auto",
        name: Optional[str] = None,
        description: Optional[str] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        pts: Optional[VideoPTS] = None,
        location: Optional[Location] = None,
        source: Optional[str] = None,
        restart: Optional[Restart] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new stream in luna-streams
        Args:
            analytics: stream analytics
            name: stream name
            description: stream description
            streamType: stream type
            downloadable: whether reference is downloadable
            timestampSource: timestamp source where pts - video labels, server - server time, frame_rate - video file
                             auto logic described in openapi
            reference: stream URL
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            pts: video pts settings
            location: location that stream belongs
            source: additional stream information
            restart: auto-restart configuration
            groupName: group name
            groupId: group identifier
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created stream id & version will be returned

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> response = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... )
        >>> print(response.statusCode)
        201
        >>> streamId = response.json["stream_id"]
        >>> response = client.getStream(streamId=streamId, raiseError=True)
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "analytics": [
                {
                    "analytic_name": "doctests_analytic",
                    "parameters": {}
                }
            ],
            "autorestart": {
                "attempt_count": 10,
                "current_attempt": 0,
                "delay": 60,
                "last_attempt_time": null,
                "restart": 0,
                "status": "disabled"
            },
            "create_time": ...,
            "data": {
                "downloadable": false,
                "pts": {
                    "start_time": 0.0
                },
                "reference": "http://reference",
                "rotation": 0,
                "timestamp_source": "auto",
                "type": "videofile"
            },
            "description": "test stream description",
            "groups": [],
            "last_error": null,
            "location": {
                "area": null,
                "city": null,
                "district": null,
                "geo_position": null,
                "house_number": null,
                "street": null
            },
            "name": "test_stream",
            "status": 0,
            "stream_id": ...,
            "version": 1
        }
        """
        body = getGeneralStreamBody(
            analytics=analytics,
            name=name,
            description=description,
            downloadable=downloadable,
            timestampSource=timestampSource,
            streamType=streamType,
            reference=reference,
            rotation=rotation,
            pts=pts,
            location=location,
            source=source,
            restart=restart,
            groupName=groupName,
            groupId=groupId,
        )
        return self.makeRequest(f"{self.baseUri}/streams", "POST", body=body, **kwargs)

    def replaceStream(
        self,
        *,
        streamId: str,
        streamType: Literal["stream", "videofile"],
        downloadable: bool = False,
        timestampSource: Literal["pts", "server", "frame_rate", "auto"] = "auto",
        reference: str,
        analytics: List[StreamAnalytic],
        name: Optional[str] = None,
        description: Optional[str] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        pts: Optional[VideoPTS] = None,
        location: Optional[Location] = None,
        source: Optional[str] = None,
        restart: Optional[Restart] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Replace an existent stream in luna-streams

        Args:
            streamId: id to put stream by
            analytics: stream analytics
            name: stream name
            description: stream description
            streamType: stream type
            downloadable: whether reference is downloadable
            timestampSource: timestamp source where pts - video labels, server - server time, frame_rate - video file
                             auto logic described in openapi
            reference: stream URL
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            pts: video pts settings
            location: location that stream belongs
            source: additional stream information
            restart: auto-restart configuration
            groupName: group name
            groupId: group identifier

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created stream id & version will be returned.
        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.replaceStream(
        ...     streamId=streamId,
        ...     streamType="videofile",
        ...     reference="http://reference.local/replaced",
        ...     name="replaced_name",
        ...     description="replaced description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ... )
        >>> print(response.statusCode)
        200
        >>> response = client.getStream(streamId=streamId, raiseError=True)
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "analytics": [
                {
                    "analytic_name": "doctests_analytic",
                    "parameters": {}
                }
            ],
            "autorestart": {
                "attempt_count": 10,
                "current_attempt": 0,
                "delay": 60,
                "last_attempt_time": null,
                "restart": 0,
                "status": "disabled"
            },
            "create_time": ...,
            "data": {
                "downloadable": false,
                "pts": {
                    "start_time": 0.0
                },
                "reference": "http://reference.local/replaced",
                "rotation": 0,
                "timestamp_source": "auto",
                "type": "videofile"
            },
            "description": "replaced description",
            "groups": [],
            "last_error": null,
            "location": {
                "area": null,
                "city": null,
                "district": null,
                "geo_position": null,
                "house_number": null,
                "street": null
            },
            "name": "replaced_name",
            "status": 0,
            "stream_id": ...,
            "version": 2
        }
        """
        body = getGeneralStreamBody(
            analytics=analytics,
            name=name,
            description=description,
            streamType=streamType,
            downloadable=downloadable,
            timestampSource=timestampSource,
            reference=reference,
            rotation=rotation,
            pts=pts,
            location=location,
            source=source,
            restart=restart,
            groupName=groupName,
            groupId=groupId,
        )
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "PUT", body=body, **kwargs)

    def getStreams(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        streamIdGte: Optional[str] = None,
        streamIdLt: Optional[str] = None,
        streamNames: Optional[List[str]] = None,
        reference: Optional[str] = None,
        statuses: Optional[List[int]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        groupName: Optional[str] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        sources: Optional[List[str]] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by filters.

        Args:
            accountId: account id
            streamIds: stream ids
            streamIdGte: stream id lower included bound
            streamIdLt: stream id upper excluded bound
            streamNames: stream names
            reference: stream reference
            statuses: stream current status values
            createTimeGte: stream create time lower including bound
            createTimeLt: stream create time upper excluding bound
            groupName: group name
            sources: additional stream information
            order: sorting order
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.getStreams(
        ...     streamIds=[streamId],
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "streams": [
                {
                    "account_id": ...,
                    "analytics": [
                        {
                            "analytic_name": "doctests_analytic",
                            "parameters": {}
                        }
                    ],
                    "autorestart": {
                        "attempt_count": 10,
                        "current_attempt": 0,
                        "delay": 60,
                        "last_attempt_time": null,
                        "restart": 0,
                        "status": "disabled"
                    },
                    "create_time": ...,
                    "data": {
                        "downloadable": false,
                        "pts": {
                            "start_time": 0.0
                        },
                        "reference": "http://reference",
                        "rotation": 0,
                        "timestamp_source": "auto",
                        "type": "videofile"
                    },
                    "description": "test stream description",
                    "groups": [],
                    "last_error": null,
                    "location": {
                        "area": null,
                        "city": null,
                        "district": null,
                        "geo_position": null,
                        "house_number": null,
                        "street": null
                    },
                    "name": "test_stream",
                    "status": 0,
                    "stream_id": ...,
                    "version": 1
                }
            ]
        }
        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "stream_id__gte": streamIdGte,
            "stream_id__lt": streamIdLt,
            "names": streamNames,
            "reference": reference,
            "statuses": statuses,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "group_name": groupName,
            "sources": sources,
            "order": order,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/streams", "GET", queryParams=queries, **kwargs)

    def countStreams(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        streamNames: Optional[List[str]] = None,
        reference: Optional[str] = None,
        statuses: Optional[List[int]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        sources: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by filters.

        Args:
            accountId: account id
            streamIds: stream ids
            streamNames: stream names
            reference: stream reference
            statuses: stream current status values
            createTimeGte: stream create time lower including bound
            createTimeLt: stream create time upper excluding bound
            sources: additional stream information

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.

        >>> import json
        >>> from uuid import uuid4
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamIds = []
        >>> for _ in range(3):
        ...     streamId = client.createStream(
        ...         streamType="videofile",
        ...         reference="http://reference",
        ...         name=str(uuid4()),
        ...         description="test stream description",
        ...         analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...         raiseError=True,
        ...     ).json["stream_id"]
        ...     streamIds.append(streamId)
        >>> response = client.countStreams(
        ...     streamIds=streamIds,
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))
        {
            "streams_count": 3
        }
        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "names": streamNames,
            "reference": reference,
            "statuses": statuses,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "sources": sources,
        }
        return self.makeRequest(f"{self.baseUri}/streams/count", "GET", queryParams=queries, **kwargs)

    def getStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with stream will be returned.
        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.getStream(streamId=streamId)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "analytics": [
                {
                    "analytic_name": "doctests_analytic",
                    "parameters": {}
                }
            ],
            "autorestart": {
                "attempt_count": 10,
                "current_attempt": 0,
                "delay": 60,
                "last_attempt_time": null,
                "restart": 0,
                "status": "disabled"
            },
            "create_time": ...,
            "data": {
                "downloadable": false,
                "pts": {
                    "start_time": 0.0
                },
                "reference": "http://reference",
                "rotation": 0,
                "timestamp_source": "auto",
                "type": "videofile"
            },
            "description": "test stream description",
            "groups": [],
            "last_error": null,
            "location": {
                "area": null,
                "city": null,
                "district": null,
                "geo_position": null,
                "house_number": null,
                "street": null
            },
            "name": "test_stream",
            "status": 0,
            "stream_id": ...,
            "version": 1
        }
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "GET", queryParams=queries, **kwargs)

    def checkStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with stream will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.checkStream(streamId=streamId)
        >>> print(response.statusCode)
        200
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "HEAD", queryParams=queries, **kwargs)

    def patchStream(
        self,
        streamId: str,
        action: str,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch stream by id.

        Args:
            streamId: stream id
            accountId: account id
            action: pause or resume

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        It's impossible to create a working doctest because the stream must be processing.

        import json
        from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        from luna3.video_manager.http_objs import StreamAnalytic
        streamId = client.createStream(
             streamType="videofile",
             reference="http://reference",
             name="test_stream",
             description="test stream description",
             analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
             raiseError=True,
        ).json["stream_id"]
        response = client.patchStream(streamId=streamId, action="pause")
        print(response.statusCode)
        204
        """
        queries = {"account_id": accountId, "action": action}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "PATCH", queryParams=queries, **kwargs)

    def deleteStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.deleteStream(streamId=streamId)
        >>> print(response.statusCode)
        204
        >>> response = client.checkStream(streamId=streamId)
        >>> print(response.statusCode)
        404
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "DELETE", queryParams=queries, **kwargs)

    def deleteStreams(
        self,
        streamIds: List[str],
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete streams by filters (OR).

        Args:
            streamIds: stream ids
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.deleteStreams(streamIds=[streamId])
        >>> print(response.statusCode)
        204
        >>> response = client.checkStream(streamId=streamId)
        >>> print(response.statusCode)
        404
        """
        body = {"stream_ids": streamIds}
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams", "DELETE", body=body, queryParams=queries, **kwargs)

    def createGroup(
        self, groupName: str, description: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new stream group.

        Args:
            groupName: group name
            description: group description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created group name and id will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> response = client.createGroup(
        ...     groupName="test_group",
        ...     description="test group description",
        ...     raiseError=True,
        ... )
        >>> print(response.statusCode)
        201
        >>> groupId = response.json["group_id"]
        >>> response = client.getGroup(groupId=groupId, raiseError=True)
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "create_time": ...,
            "description": "test group description",
            "group_id": ...,
            "group_name": "test_group"
        }
        """
        body = {"group_name": groupName}
        if description is not None:
            body["description"] = description
        payload = RequestPayload.buildJson(body=body)
        return self.makeRequest(f"{self.baseUri}/groups", "POST", body=payload, **kwargs)

    def getGroups(
        self,
        groupNames: Optional[List[str]] = None,
        groupIds: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get groups by filters.

        Args:
            accountId: account id
            groupNames: group names
            groupIds: group ids
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with groups will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> groupId = client.createGroup(
        ...     groupName="get_groups",
        ...     description="test groups description",
        ...     raiseError=True,
        ... ).json["group_id"]
        >>> response = client.getGroups(groupIds=[groupId])
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "groups": [
                {
                    "account_id": ...,
                    "create_time": ...,
                    "description": "test groups description",
                    "group_id": ...,
                    "group_name": "get_groups"
                }
            ]
        }
        """
        queries = {
            "account_id": accountId,
            "group_ids": groupIds,
            "group_names": groupNames,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/groups", "GET", queryParams=queries, **kwargs)

    def countGroups(
        self,
        groupNames: Optional[List[str]] = None,
        groupIds: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get count of groups by filter.

        Args:
            accountId: account id
            groupNames: group names
            groupIds: group ids

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with count of groups will be returned.

        >>> import json
        >>> from uuid import uuid4
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> groupIds = []
        >>> for _ in range(3):
        ...     groupId = client.createGroup(
        ...         groupName=str(uuid4()),
        ...         description="test group description",
        ...         raiseError=True,
        ...        ).json["group_id"]
        ...     groupIds.append(groupId)
        >>> response = client.countGroups(groupIds=groupIds)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))
        {
            "groups_count": 3
        }
        """
        queries = {"account_id": accountId, "group_names": groupNames, "group_ids": groupIds}
        return self.makeRequest(f"{self.baseUri}/groups/count", "GET", queryParams=queries, **kwargs)

    def getGroup(
        self, groupId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get group.

        Args:
            groupId: group id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> groupId = client.createGroup(
        ...     groupName="get_group",
        ...     description="test group description",
        ...     raiseError=True,
        ... ).json["group_id"]
        >>> response = client.getGroup(groupId=groupId)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "create_time": ...,
            "description": "test group description",
            "group_id": ...,
            "group_name": "get_group"
        }
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "GET", queryParams=queries, **kwargs)

    def patchGroup(
        self, groupId: str, description: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update group description.

        Args:
            groupId: group id
            description: group description
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> groupId = client.createGroup(
        ...     groupName="patch_group",
        ...     description="test group description",
        ...     raiseError=True,
        ... ).json["group_id"]
        >>> response = client.patchGroup(groupId=groupId, description="new description")
        >>> print(response.statusCode)
        204
        >>> response = client.getGroup(groupId=groupId)
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "create_time": ...,
            "description": "new description",
            "group_id": ...,
            "group_name": "patch_group"
        }
        """
        queries = {"account_id": accountId}
        body = {"description": description}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "PATCH", queryParams=queries, body=body, **kwargs)

    def deleteGroup(
        self, groupId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete group.

        Args:
            groupId: group id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> groupId = client.createGroup(
        ...     groupName="delete_group",
        ...     description="test group description",
        ...     raiseError=True,
        ... ).json["group_id"]
        >>> response = client.deleteGroup(groupId=groupId)
        >>> print(response.statusCode)
        204
        >>> response = client.getGroup(groupId=groupId)
        >>> print(response.statusCode)
        404
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "DELETE", queryParams=queries, **kwargs)

    def link(
        self,
        groupId: str,
        streamIds: List[str],
        action: Literal["attach", "detach"],
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Attach or detach streams to group. Optional you can set, that group and streams must belong to account.

        Args:
            groupId: group identifier
            streamIds: stream ids
            action: attach or detach
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> groupId = client.createGroup(
        ...     groupName="linked_group",
        ...     description="test group description",
        ...     raiseError=True,
        ... ).json["group_id"]
        >>> response = client.link(groupId=groupId, streamIds=[streamId], action="attach")
        >>> print(response.statusCode)
        204
        >>> response = client.getStream(streamId=streamId, raiseError=True)
        >>> print(response.json["groups"] == [groupId])
        True

        """
        queries = {"account_id": accountId}
        data = {"action": action, "group_id": groupId, "stream_ids": streamIds}
        return self.makeRequest(f"{self.baseUri}/linker", "PATCH", queryParams=queries, body=data, **kwargs)

    def getAnalytics(
        self,
        analyticIds: Optional[List[str]] = None,
        analyticIdGte: Optional[str] = None,
        analyticIdLt: Optional[str] = None,
        analyticNames: Optional[List[str]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        accountId: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get analytics by filters.

        Args:
            accountId: account id
            analyticNames: analytic names
            analyticIds: analytic ids
            order: order
            analyticIdGte: analytic id upper included boundary
            analyticIdLt: analytic id lower excluded boundary
            createTimeGte: agent create time upper included boundary
            createTimeLt: agent create time lower excluded boundary
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with groups will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> response = client.getAnalytics(analyticNames=[ANALYTIC_NAME])
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "analytics": [
                {
                    "analytic_id": ...,
                    "analytic_name": "doctests_analytic",
                    "analytic_version": 1,
                    "create_time": ...,
                    "default_parameters": {},
                    "description": "",
                    "last_update_time": ...,
                    "validation_schema": {}
                }
            ]
        }
        """
        queries = {
            "account_id": accountId,
            "analytic_ids": analyticIds,
            "names": analyticNames,
            "analytic_id__gte": analyticIdGte,
            "analytic_id__lt": analyticIdLt,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "order": order,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/analytics", "GET", queryParams=queries, **kwargs)

    def getAnalytic(
        self, analyticId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get analytic.

        Args:
            analyticId: analytic id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_ID
        >>> response = client.getAnalytic(analyticId=ANALYTIC_ID)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "analytic_id": ...",
            "analytic_name": "doctests_analytic",
            "analytic_version": 1,
            "create_time": ...,
            "default_parameters": {},
            "description": "",
            "last_update_time": ...,
            "validation_schema": {}
        }
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/analytics/{analyticId}", "GET", queryParams=queries, **kwargs)

    def getAnalyticDocumentation(
        self, analyticId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get analytic's documentation.

        Args:
            analyticId: analytic id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_ID
        >>> response = client.getAnalyticDocumentation(analyticId=ANALYTIC_ID)
        >>> print(response.statusCode)
        200
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/analytics/{analyticId}/docs", "GET", queryParams=queries, **kwargs)

    def getStreamsLogs(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        statuses: Optional[List[int]] = None,
        targets: Optional[List[str]] = None,
        logTimeLt: Optional[str] = None,
        logTimeGte: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get streams logs

        Args:
            accountId: account_id
            streamIds: stream ids
            statuses: streams statuses
            targets: log targets
            logTimeLt: log time lower excluded boundary
            logTimeGte: log time upper included boundary
            page: page
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.getStreamsLogs(
        ...     streamIds=[streamId],
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "logs": []
        }
        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "statuses": statuses,
            "targets": targets,
            "log_time__lt": logTimeLt,
            "log_time__gte": logTimeGte,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/streams/logs", "GET", queryParams=queries, **kwargs)

    def deleteStreamsLogs(
        self, logTimeLt: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete streams logs

        Args:
            logTimeLt: log time lower excluded boundary

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.


        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> response = client.deleteStreamsLogs()
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "deleted_count": ...
        }
        """
        queries = {"log_time__lt": logTimeLt}
        return self.makeRequest(f"{self.baseUri}/streams/logs", "DELETE", queryParams=queries, **kwargs)

    def streamsWsConnect(
        self,
        streamId: str,
        accountId: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
        asyncRequest: Optional[bool] = None,
        messageReceiveTimeout: Optional[Real] = None,
    ) -> Union[aiohttp.client._WSRequestContextManager, contextmanager]:  # pylint: disable=W0212
        """
        Create ws-connection to receive real time data from stream analytics

        Args:
            streamId: stream id
            accountId: account id to connect to
            login: account login
            password: account password
            token: token

            asyncRequest: execution in asynchronous mode, disabled by default
            messageReceiveTimeout: message receive timeout (time between start waiting message and message appearance)
        Returns:
            context manager with connection to luna-video-manager

        >>> import json# doctest: +SKIP
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client    # doctest: +SKIP
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME    # doctest: +SKIP
        >>> from luna3.video_manager.http_objs import StreamAnalytic    # doctest: +SKIP
        >>> response = client.createStream(
        ...     streamType="videofile",
        ...     reference="http://reference",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... )    # doctest: +SKIP
        >>> streamId = response.json["stream_id"]    # doctest: +SKIP
        >>> with client.streamsWsConnect(streamId=streamId, messageReceiveTimeout=5) as ws:    # doctest: +SKIP
        ...     WS_RESP = json.loads(ws.recv())    # doctest: +SKIP
        >>> print(ws.status)    # doctest: +SKIP
        101
        >>> print(json.dumps(WS_RESP, sort_keys=True, indent=4))     # doctest: +SKIP
        {
          "stream_status": "in_progress",
          "error": null,
          "analytics_results": {
            "crowd": {
              "track_id": ...,
              "event_id": ...,
              "people_count": ...,
              "time_offset": ...,
              "event_status": "started",
              "people_coordinates": ...
            }
          }
        }
        """
        headers = self.getAuthHeader(accountId=accountId, login=login, password=password, token=token)
        return super().wsConnect(
            path=f"/streams/{streamId}/ws",
            headers=headers,
            queryParams={},
            messageTimeout=messageReceiveTimeout,
            asyncRequest=asyncRequest,
        )

    def getRestream(
        self,
        streamId: str,
        restreamType: Literal["hls"] = "hls",
        restreamQuality: Optional[int] = None,
        **kwargs,
    ):
        """
        Get restream

        Args:
            streamId: stream id
            restreamType: restream type
            restreamQuality: restream quality
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with stream will be returned.
        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.lunavl.mixins.doctests.config import ANALYTIC_NAME
        >>> from luna3.video_manager.http_objs import StreamAnalytic
        >>> streamId = client.createStream(
        ...     streamType="stream",
        ...     reference="rtsp://dev02:5554/tenvision_1",
        ...     name="test_stream",
        ...     description="test stream description",
        ...     analytics=[StreamAnalytic(name=ANALYTIC_NAME)],
        ...     raiseError=True,
        ... ).json["stream_id"]
        >>> response = client.getRestream(streamId=streamId)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "quality": ...,
            "token": ...,
            "type": ...,
            "url": ...
        }
        """
        queryParams = {}
        if restreamType:
            queryParams["type"] = restreamType
        if restreamQuality:
            queryParams["quality"] = restreamQuality
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}/restream", "GET", queryParams=queryParams, **kwargs)
