"""
Module contains backward compatible match objects for luna-events
"""

from dataclasses import MISSING, dataclass, field, fields
from typing import Any, ClassVar, Dict, List, Optional, Union

from stringcase import snakecase

from luna3.common.http_objs import CLOTHING_COLOR, PRIMITIVE_CLOTHING_COLOR, _aliasproperty

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

EVENT_TARGET = Literal[  # pylint: disable-msg=invalid-name
    "account_id",
    "create_time",
    "end_time",
    "event_id",
    "handler_id",
    "external_id",
    "source",
    "stream_id",
    "top_match",
    "match_result",
    "face_detections",
    "face_id",
    "attach_result",
    "gender",
    "age",
    "emotion",
    "ethnic_group",
    "liveness",
    "deepfake",
    "body_detections",
    "body_basic_attributes",
    "upper_body",
    "lower_body",
    "accessories",
    "tags",
    "user_data",
    "location",
    "mask",
    "track_id",
    "meta",
]


@dataclass
class BaseClass:
    """
    Base class with default method `toDict`.
    """

    # cached aliases storage {<class id>: {<CamelCase name>: <SnakeCase name>}}. For every class new aliases
    _cachedAliases: ClassVar[Dict[str, Dict[str, str]]] = {}

    @_aliasproperty
    def _aliases(cls) -> Dict[str, str]:  # pylint: disable-msg=E0213
        """Aliases for dataclass fields in snake case."""
        aliases = cls._cachedAliases.get(id(cls))  # pylint: disable-msg=no-member
        if aliases is None:
            aliases = {clsField.name: snakecase(clsField.name) for clsField in fields(cls)}
            cls._cachedAliases[id(cls)] = aliases  # pylint: disable-msg=no-member
        return aliases

    def asDict(self) -> Dict:
        """
        Recursively cast class to dict.

        Returns:
            self as dict without empty fields
        """

        def parseValue(value: Union[BaseClass, List, Any]) -> Any:
            """
            Recursively parse value to json representation.

            Args:
                value: field value

            Returns:
                parsed value
            """
            if isinstance(value, BaseClass):
                return value.asDict()
            if isinstance(value, list):
                return [parseValue(item) for item in value]
            return value

        res = {}
        for fieldObj in fields(self):
            fieldValue = getattr(self, fieldObj.name)
            if fieldObj.default is not MISSING:
                if fieldValue is not None:
                    res[self._aliases[fieldObj.name]] = parseValue(fieldValue)
                else:
                    # skip optional nullable fields
                    continue
            else:
                res[self._aliases[fieldObj.name]] = parseValue(fieldValue)
        return res


@dataclass
class Descriptor(BaseClass):
    """
    Descriptor class

    Attributes:
        descriptor: raw descriptor
        descriptorVersion: descriptor version
    """

    descriptor: bytes = field(repr=False)
    descriptorVersion: int


@dataclass
class Rect(BaseClass):
    """
    Rect class

    Attributes:
        height: height
        width: width
        x: x coordinate
        y: y coordinate
    """

    height: int
    width: int
    x: int  # pylint: disable=C0103
    y: int  # pylint: disable=C0103


@dataclass
class EmotionsEstimations(BaseClass):
    """
    Emotions estimations class

    Attributes:
        anger: anger estimation value
        disgust: disgust estimation value
        fear: fear estimation value
        happiness: happiness estimation value
        neutral: neutral estimation value
        sadness: sadness estimation value
        surprise: surprise estimation value
    """

    anger: float
    disgust: float
    fear: float
    happiness: float
    neutral: float
    sadness: float
    surprise: float


@dataclass
class Emotions(BaseClass):
    """
    Emotions class

    Attributes:
        estimations: emotions estimations
        predominantEmotion: predominant emotion name
    """

    estimations: EmotionsEstimations
    predominantEmotion: str


@dataclass
class MaskEstimations(BaseClass):
    """
    Mask estimations class

    Attributes:
        medicalMask: medical mask estimation
        missing: missing estimation
        occluded: occluded estimation
    """

    medicalMask: float
    missing: float
    occluded: float


@dataclass
class Mask(BaseClass):
    """
    Mask class

    Attributes:
        estimations: mask estimations
        predominantMask: predominant mask name
    """

    estimations: MaskEstimations
    predominantMask: str


@dataclass
class LivenessEstimations(BaseClass):
    """
    Liveness estimations

    Attributes:
        score: liveness score
        quality: liveness quality
    """

    score: float
    quality: float


@dataclass
class Liveness(BaseClass):
    """
    Liveness class

    Attributes:
        prediction: liveness prediction
        estimations: liveness estimations
    """

    estimations: LivenessEstimations
    prediction: str


@dataclass
class FaceOcclusionEstimations(BaseClass):
    """
    Face occlusion estimations

    Attributes:
        overallScore: overlapped face area
        hairScore:  overlapped face area by hair
        lowerFaceScore: score of an overlapped area of a lower face
        noseScore: score of an overlapped area of a nose
        mouthScore: score of an overlapped area of a mouth
        leftEyeScore: score of an overlapped area of a left eye
        rightEyeScore: score of an overlapped area of a right eye
        foreheadScore: score of an overlapped area of a forehead
    """

    overallScore: float
    hairScore: float
    lowerFaceScore: float
    noseScore: float
    mouthScore: float
    leftEyeScore: float
    rightEyeScore: float
    foreheadScore: float


@dataclass
class FaceOverlaps(BaseClass):
    """
    Face occlusion area prediction

    Attributes:
        lowerFaceOccluded:  lower face is overlapped or not
        mouthOccluded: mouth is overlapped or not
        noseOccluded: nose is overlapped or not
        leftEyeOccluded: left eye is overlapped or not
        rightEyeOccluded: right eye is overlapped or not
        foreheadOccluded: forehead is overlapped or not
    """

    lowerFaceOccluded: int
    mouthOccluded: int
    noseOccluded: int
    rightEyeOccluded: int
    leftEyeOccluded: int
    foreheadOccluded: int


@dataclass
class FaceAreaOcclusion(BaseClass):
    """
    Face occlusion class

    Attributes:
        prediction: face occlusion prediction
        estimations: face occlusion estimations
        faceOcclusion:
    """

    prediction: float
    estimations: FaceOcclusionEstimations
    faceOcclusion: FaceOverlaps


@dataclass
class Deepfake(BaseClass):
    """
    Deepfake class

    Attributes:
        prediction: deepfake prediction
        score: deepfake score
    """

    prediction: str
    score: float


@dataclass
class FaceDetection(BaseClass):
    """
    Face detection class

    Attributes:
        rect: face detection rect
    """

    rect: Optional[Rect] = None


@dataclass
class FaceSample(BaseClass):
    """
    Face sample class

    Attributes:
        sampleId: face sample id
        detection: face sample detection
    """

    sampleId: str
    detection: Optional[FaceDetection] = None


@dataclass
class BodyDetection(BaseClass):
    """
    Body detection class

    Attributes:
        rect: body detection rect
        bodyAttributes: body attributes
    """

    rect: Optional[Rect] = None


@dataclass
class BodySample(BaseClass):
    """
    Body sample class

    Attributes:
        sampleId: body sample id
        detection: body sample detection
    """

    sampleId: str
    detection: Optional[BodyDetection] = None


@dataclass
class DetectionSamples(BaseClass):
    """
    Detection samples class. The face and the body refer to the same human.

    Attributes:
        face: face sample
        body: body sample
    """

    face: Optional[FaceSample] = None
    body: Optional[BodySample] = None

    def asDict(self) -> dict:
        """
        Cast self into dict. Used by `ujson.dumps`.

        Returns:
            self as dict
        """
        return {"face": self.face.asDict() if self.face else None, "body": self.body.asDict() if self.body else None}


@dataclass
class EventDetection(BaseClass):
    """
    Event detection class

    Attributes:
        filename: source image filename
        samples: samples
        detectTime: time in format RFC 3339
        imageOrigin: the image url - source of the detection
        detectTs: user-defined timestamp relative to something, such as the start of a video
    """

    filename: str
    samples: DetectionSamples
    detectTime: str
    imageOrigin: Optional[str] = None
    detectTs: Optional[float] = None


@dataclass
class EventFace(BaseClass):
    """
    Event face class

    Attributes:
        faceId: created face id
        lists: attach result (list ids)
    """

    faceId: str
    lists: List[str]


@dataclass
class BasicEthnicities(BaseClass):
    """
    Basic ethnicities class

    Attributes:
        predominantEthnicity: predominant ethnicity name
    """

    predominantEthnicity: str


@dataclass
class BasicAttributes(BaseClass):
    """
    Basic attributes class

    Attributes:
        age: age
        ethnicities: ethnicities
        gender: gender (0 - female, 1 - male)
    """

    age: int
    ethnicities: BasicEthnicities
    gender: int


@dataclass
class EventFaceAttributes(BaseClass):
    """
    Event face attributes class

    Attributes:
        basicAttributes: basic attributes
        descriptorData: descriptor data
    """

    basicAttributes: Optional[BasicAttributes] = None
    descriptorData: Optional[Descriptor] = None


@dataclass
class EventBodyAttributes(BaseClass):
    """
    Event face attributes class

    Attributes:
        descriptorData: descriptor data
    """

    descriptorData: Optional[Descriptor] = None


@dataclass
class GeoPosition(BaseClass):
    """
    Geo position class

    Attributes:
        latitude: latitude in degrees [-90..90]
        longitude: longitude in degrees [-180..180]
    """

    latitude: float
    longitude: float


@dataclass
class Location(BaseClass):
    """
    Location class

    Attributes:
        area: area
        city: city
        district: district
        street: street
        houseNumber: house number
        geoPosition: geo position
    """

    area: Optional[str] = None
    city: Optional[str] = None
    district: Optional[str] = None
    street: Optional[str] = None
    houseNumber: Optional[str] = None
    geoPosition: Optional[GeoPosition] = None


@dataclass
class FaceCandidate(BaseClass):
    """
    Face candidate class

    Attributes:
        faceId: face id
        createTime: create time
        externalId: external id
        userData: user data
    """

    faceId: Optional[str] = None
    createTime: Optional[str] = None
    externalId: Optional[str] = None
    userData: Optional[str] = None


@dataclass
class EventCandidate(BaseClass):
    """
    Event candidate class

    Attributes:
        eventId: event id
        source: source
        streamId: stream id
        handlerId: handler id
        externalId: external id
        userData: user data
        createTime: create time
    """

    eventId: Optional[str] = None
    source: Optional[str] = None
    streamId: Optional[str] = None
    handlerId: Optional[str] = None
    externalId: Optional[str] = None
    userData: Optional[str] = None
    createTime: Optional[str] = None


@dataclass
class MatchCandidate(BaseClass):
    """
    Match candidate class.
    If matched against face - face must be specified.
    If matched against event - event must be specified.

    Attributes:
        similarity: similarity
        face: candidate-face
        event: candidate-event
    """

    similarity: float
    face: Optional[FaceCandidate] = None
    event: Optional[EventCandidate] = None

    def __post_init__(self):
        if (self.face is None) is (self.event is None):
            raise ValueError(str(self))


@dataclass
class EventMatch(BaseClass):
    """
    Event match class

    Attributes:
        label: match label
        candidates: match candidates
    """

    label: str
    candidates: List[MatchCandidate]


@dataclass
class FaceAggregateEstimationsAttributes(BaseClass):
    """
    Event aggregate estimations class

    Attributes:
        liveness: liveness detection
        deepfake: deepfake detection
        emotions: face emotions
        mask: face mask
    """

    liveness: Optional[Liveness] = None
    deepfake: Optional[Deepfake] = None
    emotions: Optional[Emotions] = None
    mask: Optional[Mask] = None


@dataclass
class FaceAggregateEstimations(BaseClass):
    """
    Event face aggregate estimations class

    Attributes:
        attributes: face aggregate estimations attributes
    """

    attributes: FaceAggregateEstimationsAttributes


@dataclass
class BodyBasicAttributes(BaseClass):
    """
    Event body basic attributes

    Attributes:
        apparentAge: apparent age
        apparentGender: apparent gender (0 - female, 1 - male, 2 - undefined)
    """

    apparentAge: int
    apparentGender: Literal[0, 1, 2]


@dataclass
class BodyAttributeState(BaseClass):
    """
    Event body attribute state estimation.
    Attributes:
        state: attribute state (0 - absent, 1 - present, 2 - undefined)
    """

    state: Literal[0, 1, 2]


@dataclass
class BodyAttributeHeadwear(BodyAttributeState):
    """
    Body attribute headwear estimation.
    """

    apparentColor: PRIMITIVE_CLOTHING_COLOR = None


class BodyAttributeBackpack(BodyAttributeState):
    """Body attribute backpack estimation."""


@dataclass
class BodyAttributeSleeve(BaseClass):
    """
    Event body attribute sleeve estimation.
    Attributes:
        length: sleeve length
    """

    length: Literal["short", "long", "undefined"]


@dataclass
class BodyAttributeClothing(BaseClass):
    """
    Event body attribute upper clothing estimation.
    Attributes:
        colors: upper clothing colors
    """

    colors: List[CLOTHING_COLOR]


@dataclass
class BodyUpperAttributes(BaseClass):
    """
    Event body upper attributes

    Attributes:
        headwear: headwear attributes
        sleeveLength: sleeve attributes
        upperClothing: upper clothing attributes
    """

    headwear: BodyAttributeHeadwear
    sleeve: BodyAttributeSleeve
    upperClothing: BodyAttributeClothing


@dataclass
class BodyAttributeLowerGarment(BaseClass):
    """
    Event body attribute upper clothing estimation.
    """

    type: Literal["undefined", "trousers", "shorts", "skirt"]
    colors: List[CLOTHING_COLOR]


@dataclass
class BodyAttributeShoes(BaseClass):
    """
    Event body attribute shoes estimation.
    """

    apparentColor: PRIMITIVE_CLOTHING_COLOR


@dataclass
class LowerBodyAttributes(BaseClass):
    """
    Event lower body attributes
    """

    lowerGarment: BodyAttributeLowerGarment
    shoes: BodyAttributeShoes


@dataclass
class BodyAccessories(BaseClass):
    """
    Event body accessories

    Attributes:
        backpack: backpack attributes
    """

    backpack: BodyAttributeBackpack


@dataclass
class BodyExtraAttributes(BaseClass):
    """
    Event body extra attributes

    Attributes:
        basicAttributes: body basic attributes
        upperBody: upper body attributes
        lowerBody: lower body attributes
        accessories: body accessories
    """

    basicAttributes: Optional[BodyBasicAttributes] = None
    upperBody: Optional[BodyUpperAttributes] = None
    lowerBody: Optional[LowerBodyAttributes] = None
    accessories: Optional[BodyAccessories] = None


@dataclass
class BodyAggregateEstimations(BaseClass):
    """
    Event body aggregate estimations class

    Attributes:
        attributes: body aggregated attributes
    """

    attributes: BodyExtraAttributes


@dataclass
class AggregateEstimations(BaseClass):
    """
    Event aggregated estimations class

    Attributes:
        face: face aggregated estimations
        body: body aggregated estimations
    """

    face: Optional[FaceAggregateEstimations] = None
    body: Optional[BodyAggregateEstimations] = None


@dataclass
class Event(BaseClass):
    """
    Core event class

    Attributes:
        eventId: event id
        createTime: create time
        handlerId: handle id
        endTime: end time
        detections: detections
        faceAttributes: face attributes
        bodyAttributes: body attributes
        externalId: external id
        face: created face
        location: location
        matches: matches
        source: source
        streamId: stream id
        tags: tags
        trackId: track id
        userData: user data
        aggregateEstimations: aggregated estimations
    """

    eventId: str
    createTime: str
    handlerId: str
    endTime: Optional[str] = None
    detections: Optional[List[EventDetection]] = None
    faceAttributes: Optional[EventFaceAttributes] = None
    bodyAttributes: Optional[EventBodyAttributes] = None
    externalId: Optional[str] = None
    face: Optional[EventFace] = None
    location: Optional[Location] = None
    matches: Optional[List[EventMatch]] = None
    source: Optional[str] = None
    streamId: Optional[str] = None
    tags: Optional[List[str]] = None
    trackId: Optional[str] = None
    userData: Optional[str] = None
    aggregateEstimations: Optional[AggregateEstimations] = None
    meta: Optional[dict] = None


@dataclass
class GeneralEvent(BaseClass):
    """
    Core event class

    Attributes:
        accountId: account id
        eventType: event type
        createTime: create time
        endTime: end time
        source: source
        eventId: event id
        streamId: stream id
        trackId: track id
        location: location
        eventContent: general event data
    """

    accountId: str
    eventType: str
    eventId: str
    createTime: str
    endTime: Optional[str] = None
    source: Optional[str] = None
    streamId: Optional[str] = None
    trackId: Optional[str] = None
    location: Optional[Location] = None
    eventContent: Optional[dict] = None

    def asDict(self):
        res = {
            "account_id": self.accountId,
            "event_type": self.eventType,
            "event_create_time": self.createTime,
        }
        if self.endTime is not None:
            res["event_end_time"] = self.endTime

        res["event"] = self.eventContent or {}
        res["event"]["event_id"] = self.eventId
        if self.source is not None:
            res["event"]["source"] = self.source
        if self.streamId is not None:
            res["event"]["stream_id"] = self.streamId
        if self.trackId is not None:
            res["event"]["track_id"] = self.trackId
        if self.location is not None:
            res["event"]["location"] = self.location.asDict()
        return res
