"""
Luna-streams-retranslator client module
"""

from numbers import Real
from typing import Literal, Optional

import aiohttp

from ..common.base_api import BaseHTTPAPI


class StreamsRetranslatorApi(BaseHTTPAPI):
    """
    Class for request to luna-streams-retranslator.

    """

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5250",
        api: Optional[int] = 1,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        **kwargs,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )

    def registerStream(
        self,
        accountId: str,
        url: str,
        streamId: str,
        restreamType: Literal["hls", "rtsp"] = "hls",
        restreamQuality: Optional[int] = None,
        **kwargs,
    ):
        """
        Register stream

        Args:
            accountId: account id
            url: stream url
            streamId: stream id
            restreamType: restream type
            restreamQuality: restream quality
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        body = {"url": url, "type": restreamType}
        if restreamQuality is not None:
            body["quality"] = restreamQuality
        return self.makeRequest(
            f"{self.baseUri}/streams/{streamId}", "PUT", body=body, headers={"Luna-Account-Id": accountId}, **kwargs
        )

    def getRestreams(
        self,
        streamId: str,
        restreamType: Optional[Literal["hls", "rtsp"]] = None,
        restreamQuality: Optional[int] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ):
        """
        Get restreams

        Args:
            accountId: account id
            streamId: stream id
            restreamType: restream type
            restreamQuality: restream quality
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queryParams = {}
        if accountId:
            queryParams["account_id"] = accountId
        if restreamType:
            queryParams["type"] = restreamType
        if restreamQuality:
            queryParams["quality"] = restreamQuality
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "GET", queryParams=queryParams, **kwargs)

    def deleteRestreams(
        self,
        streamId: str,
        accountId: Optional[str] = None,
        restreamType: Optional[Literal["hls", "rtsp"]] = None,
        restreamQuality: Optional[int] = None,
        **kwargs,
    ):
        """
        Delete restream(s)

        Args:
            accountId: account id
            streamId: stream id
            restreamType: restream type
            restreamQuality: restream quality

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queryParams = {}
        if accountId:
            queryParams["account_id"] = accountId
        if restreamType:
            queryParams["type"] = restreamType
        if restreamQuality:
            queryParams["quality"] = restreamQuality
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "DELETE", queryParams=queryParams, **kwargs)
