""" Some structures for Streams Api. """

from dataclasses import asdict, dataclass
from typing import List, Optional

from stringcase import snakecase


@dataclass
class GeoPosition:
    """Geo position class"""

    latitude: float
    longitude: float


@dataclass
class Location:
    """Location structure."""

    area: Optional[str] = None
    city: Optional[str] = None
    district: Optional[str] = None
    street: Optional[str] = None
    houseNumber: Optional[str] = None
    geoPosition: Optional[GeoPosition] = None

    def asDict(self):
        """Creates dict representation."""
        return {snakecase(field): value for field, value in asdict(self).items() if value is not None}


@dataclass
class Restart:
    """Restart configuration structure."""

    restart: Optional[bool] = False
    attemptCount: Optional[int] = 10
    delay: Optional[int] = 60

    def asDict(self):
        """Creates dict representation."""
        return {snakecase(field): int(value) for field, value in asdict(self).items() if value is not None}


@dataclass
class Agent:
    """Video manager agent structure"""

    name: str
    maxStreamCount: int
    port: int
    apiVersion: int
    analyticNames: List[str]
    description: Optional[str] = None

    def asDict(self) -> dict:
        """Creates dict representation."""
        res = {
            "name": self.name,
            "port": self.port,
            "api_version": self.apiVersion,
            "max_stream_count": self.maxStreamCount,
            "analytic_names": self.analyticNames,
        }
        if self.description is not None:
            res["description"] = self.description
        return res


@dataclass
class Analytic:
    """Video manager analytic structure"""

    analyticName: str
    version: Optional[int] = None
    description: Optional[str] = None
    documentation: Optional[str] = None
    defaultParameters: Optional[dict] = None

    def asDict(self) -> dict:
        """Creates dict representation"""
        res = {"analytic_name": self.analyticName}
        if self.version is not None:
            res["analytic_version"] = self.version
        if self.description is not None:
            res["description"] = self.description
        if self.documentation is not None:
            res["documentation"] = self.documentation
        if self.defaultParameters is not None:
            res["default_parameters"] = self.defaultParameters
        return res


@dataclass
class StreamAnalytic:
    """Video manager stream analytic"""

    name: str
    parameters: Optional[dict] = None

    def asDict(self) -> dict:
        """Create dictionary representation"""
        res = {"analytic_name": self.name}
        if self.parameters is not None:
            res["parameters"] = self.parameters
        return res
