"""Module contains utils for Video Manager"""

from typing import List, Literal, Optional

from luna3.public.sdk import VideoPTS
from luna3.video_manager.http_objs import Location, Restart, StreamAnalytic


def getGeneralStreamBody(
    *,
    streamType: Literal["stream", "videofile"],
    reference: str,
    analytics: Optional[List[StreamAnalytic]],
    downloadable: bool = False,
    timestampSource: Literal["pts", "server", "frame_rate", "auto"] = "auto",
    name: Optional[str] = None,
    description: Optional[str] = None,
    rotation: Optional[Literal[0, 90, 180, 270]] = None,
    pts: Optional[VideoPTS] = None,
    location: Optional[Location] = None,
    source: Optional[str] = None,
    restart: Optional[Restart] = None,
    groupName: Optional[str] = None,
    groupId: Optional[str] = None,
) -> dict:
    """
    Build template body for stream creation request.

    Args:
        analytics: stream analytics
        name: stream name
        description: stream description
        streamType: stream type
        downloadable: whether reference is downloadable
        timestampSource: timestamp source where pts - video labels, server - server time, frame_rate - video file
                         auto logic described in openapi
        reference: stream URL
        rotation: angle of camera frame rotation (0, 90, 180 or 270)
        pts: video pts settings
        location: location that stream belongs
        restart: auto-restart configuration
        groupName: group name
        groupId: group identifier

    Returns:
        request body as dict.

    """

    body = {
        "data": {
            "type": streamType,
            "reference": reference,
            "downloadable": downloadable,
            "timestamp_source": timestampSource,
        },
        "analytics": [row.asDict() for row in analytics],
    }
    if name:
        body["name"] = name
    if description:
        body["description"] = description
    if rotation:
        body["data"]["rotation"] = rotation
    if pts:
        body["data"]["pts"] = pts.asDict()
    if restart:
        body["autorestart"] = restart.asDict()
    if location:
        body["location"] = location.asDict()
    if source is not None:
        body["source"] = source
    if groupName:
        body["group_name"] = groupName
    if groupId:
        body["group_id"] = groupId

    return body
