"""
Luna-video-manager client module
"""

from contextlib import contextmanager
from numbers import Real
from typing import Awaitable, Dict, List, Optional, Union

import aiohttp
from websocket import create_connection

from ..common.base_api import BaseHTTPAPI
from ..common.luna_response import LunaResponse
from ..common.requests import RequestPayload
from ..common.ws_base import WSBase
from ..public.sdk import VideoPTS
from .http_objs import Agent, Analytic, Location, Restart, StreamAnalytic
from .utils import getGeneralStreamBody

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal


class VideoManagerApi(BaseHTTPAPI, WSBase):
    """
    Class for request to luna-video-manager.

    """

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5230",
        api: Optional[int] = 1,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        messageReceiveTimeout: Optional[Real] = None,
        accountId: Optional[str] = None,
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        **kwargs,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )
        self.accountId = accountId
        self.messageReceiveTimeout = messageReceiveTimeout

    def getAccountIdAsDict(self, accountId: Optional[str] = None) -> Dict[str, str]:
        """
        Get final account id header.

        Args:
            accountId: account id. Pattern:
                ^[0-9]{10},[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}$
        Returns:
            empty dict if both accountId and self.accountId are None or
                return dict with accountId or self.accountId
        """
        accountId = accountId if accountId is not None else self.accountId
        if accountId is not None:
            return {"Luna-Account-Id": accountId}
        return {}

    def createStream(
        self,
        *,
        accountId: str,
        streamType: Literal["stream", "videofile"],
        downloadable: bool = False,
        timestampSource: Literal["pts", "server", "frame_rate", "auto"] = "auto",
        reference: str,
        analytics: List[StreamAnalytic],
        name: Optional[str] = None,
        description: Optional[str] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        pts: Optional[VideoPTS] = None,
        location: Optional[Location] = None,
        source: Optional[str] = None,
        restart: Optional[Restart] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new stream in luna-streams
        Args:
            accountId: stream account id
            analytics: stream analytics
            name: stream name
            description: stream description
            streamType: stream type
            downloadable: whether reference is downloadable
            timestampSource: timestamp source where pts - video labels, server - server time, frame_rate - video file
                             auto logic described in openapi
            reference: stream URL
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            pts: video pts settings
            location: location that stream belongs
            source: additional stream information
            restart: auto-restart configuration
            groupName: group name
            groupId: group identifier
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created stream id & version will be returned
        """
        body = getGeneralStreamBody(
            analytics=analytics,
            name=name,
            downloadable=downloadable,
            timestampSource=timestampSource,
            description=description,
            streamType=streamType,
            reference=reference,
            rotation=rotation,
            pts=pts,
            location=location,
            source=source,
            restart=restart,
            groupName=groupName,
            groupId=groupId,
        )
        return self.makeRequest(
            f"{self.baseUri}/streams", "POST", body=body, headers={"Luna-Account-Id": accountId}, **kwargs
        )

    def replaceStream(
        self,
        *,
        streamId: str,
        accountId: str,
        streamType: Literal["stream", "videofile"],
        downloadable: bool = False,
        timestampSource: Literal["pts", "server", "frame_rate", "auto"] = "auto",
        reference: str,
        analytics: List[StreamAnalytic],
        name: Optional[str] = None,
        description: Optional[str] = None,
        rotation: Optional[Literal[0, 90, 180, 270]] = None,
        pts: Optional[VideoPTS] = None,
        location: Optional[Location] = None,
        source: Optional[str] = None,
        restart: Optional[Restart] = None,
        groupName: Optional[str] = None,
        groupId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Replace an existent stream in luna-streams

        Args:
            streamId: id to put stream by
            accountId: stream account id
            analytics: stream analytics
            name: stream name
            description: stream description
            streamType: stream type
            downloadable: whether reference is downloadable
            timestampSource: timestamp source where pts - video labels, server - server time, frame_rate - video file
                             auto logic described in openapi
            reference: stream URL
            rotation: angle of camera frame rotation (0, 90, 180 or 270)
            pts: video pts settings
            location: location that stream belongs
            source: additional stream information
            restart: auto-restart configuration
            groupName: group name
            groupId: group identifier

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created stream id & version will be returned.

        """
        body = getGeneralStreamBody(
            analytics=analytics,
            name=name,
            description=description,
            streamType=streamType,
            reference=reference,
            rotation=rotation,
            pts=pts,
            location=location,
            source=source,
            restart=restart,
            downloadable=downloadable,
            timestampSource=timestampSource,
            groupName=groupName,
            groupId=groupId,
        )
        return self.makeRequest(
            f"{self.baseUri}/streams/{streamId}", "PUT", body=body, headers={"Luna-Account-Id": accountId}, **kwargs
        )

    def getStreams(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        streamIdGte: Optional[str] = None,
        streamIdLt: Optional[str] = None,
        streamNames: Optional[List[str]] = None,
        reference: Optional[str] = None,
        statuses: Optional[List[int]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        groupName: Optional[str] = None,
        sources: Optional[List[str]] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by filters.

        Args:
            accountId: account id
            streamIds: stream ids
            streamIdGte: stream id lower included bound
            streamIdLt: stream id upper excluded bound
            streamNames: stream names
            reference: stream reference
            statuses: stream current status values
            createTimeGte: stream create time lower including bound
            createTimeLt: stream create time upper excluding bound
            groupName: group name
            sources: additional stream information
            order: sorting order
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.

        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "stream_id__gte": streamIdGte,
            "stream_id__lt": streamIdLt,
            "names": streamNames,
            "reference": reference,
            "statuses": statuses,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "group_name": groupName,
            "order": order,
            "sources": sources,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/streams", "GET", queryParams=queries, **kwargs)

    def countStreams(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        streamNames: Optional[List[str]] = None,
        reference: Optional[str] = None,
        statuses: Optional[List[int]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        sources: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by filters.

        Args:
            accountId: account id
            streamIds: stream ids
            streamNames: stream names
            reference: stream reference
            statuses: stream current status values
            createTimeGte: stream create time lower including bound
            createTimeLt: stream create time upper excluding bound
            sources: additional stream information

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with streams will be returned.

        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "names": streamNames,
            "reference": reference,
            "statuses": statuses,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "sources": sources,
        }
        return self.makeRequest(f"{self.baseUri}/streams/count", "GET", queryParams=queries, **kwargs)

    def getStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with stream will be returned.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "GET", queryParams=queries, **kwargs)

    def checkStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with stream will be returned.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "HEAD", queryParams=queries, **kwargs)

    def patchStream(
        self,
        streamId: str,
        action: str,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch stream by id.

        Args:
            streamId: stream id
            accountId: account id
            action: pause or resume

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"account_id": accountId, "action": action}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "PATCH", queryParams=queries, **kwargs)

    def deleteStream(
        self, streamId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete stream by id.

        Args:
            streamId: stream id
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}", "DELETE", queryParams=queries, **kwargs)

    def deleteStreams(
        self,
        streamIds: List[str],
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete streams by filters (OR).

        Args:
            streamIds: stream ids
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        body = {"stream_ids": streamIds}
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/streams", "DELETE", body=body, queryParams=queries, **kwargs)

    def createGroup(
        self, groupName: str, accountId: str, description: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new stream group.

        Args:
            accountId: account id
            groupName: group name
            description: group description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created group name and id will be returned.
        """
        body = {"group_name": groupName}
        if description is not None:
            body["description"] = description
        payload = RequestPayload.buildJson(body=body)
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/groups", "POST", body=payload, **kwargs)

    def getGroups(
        self,
        groupNames: Optional[List[str]] = None,
        groupIds: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get groups by filters.

        Args:
            accountId: account id
            groupNames: group names
            groupIds: group ids
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with groups will be returned.
        """
        queries = {
            "account_id": accountId,
            "group_ids": groupIds,
            "group_names": groupNames,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/groups", "GET", queryParams=queries, **kwargs)

    def countGroups(
        self,
        groupNames: Optional[List[str]] = None,
        groupIds: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get count of groups by filter.

        Args:
            accountId: account id
            groupNames: group names
            groupIds: group ids

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with count of groups will be returned.
        """
        queries = {"account_id": accountId, "group_names": groupNames, "group_ids": groupIds}
        return self.makeRequest(f"{self.baseUri}/groups/count", "GET", queryParams=queries, **kwargs)

    def getGroup(
        self, groupId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get group.

        Args:
            groupId: group id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "GET", queryParams=queries, **kwargs)

    def patchGroup(
        self, groupId: str, description: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update group description.

        Args:
            groupId: group id
            description: group description
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        body = {"description": description}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "PATCH", queryParams=queries, body=body, **kwargs)

    def deleteGroup(
        self, groupId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete group.

        Args:
            groupId: group id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/groups/{groupId}", "DELETE", queryParams=queries, **kwargs)

    def link(
        self,
        groupId: str,
        streamIds: List[str],
        action: Literal["attach", "detach"],
        accountId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Attach or detach streams to group. Optional you can set, that group and streams must belong to account.

        Args:
            groupId: group identifier
            streamIds: stream ids
            action: attach or detach
            accountId: account id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"account_id": accountId}
        data = {"action": action, "group_id": groupId, "stream_ids": streamIds}
        return self.makeRequest(f"{self.baseUri}/linker", "PATCH", queryParams=queries, body=data, **kwargs)

    def createAgent(self, accountId: str, agent: Agent, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new agent.

        Args:
            accountId: account id
            agent: agent structure

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created group name and id will be returned.
        """
        payload = RequestPayload.buildJson(body=agent.asDict())
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/agents", "POST", body=payload, **kwargs)

    def getAgents(
        self,
        agentIds: Optional[List[str]] = None,
        agentIdGte: Optional[str] = None,
        agentIdLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        agentNames: Optional[List[str]] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        accountId: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get agents by filters.

        Args:
            accountId: account id
            agentNames: agent names
            agentIds: agent ids
            order: order
            agentIdGte: agent id upper included boundary
            agentIdLt: agent id lower excluded boundary
            createTimeGte: agent create time upper included boundary
            createTimeLt: agent create time lower excluded boundary
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with groups will be returned.
        """
        queries = {
            "account_id": accountId,
            "agent_ids": agentIds,
            "names": agentNames,
            "agent_id__gte": agentIdGte,
            "agent_id__lt": agentIdLt,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "order": order,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/agents", "GET", queryParams=queries, **kwargs)

    def getAgent(
        self, agentId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get agent.

        Args:
            agentId: agent id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/agents/{agentId}", "GET", queryParams=queries, **kwargs)

    def checkAgent(
        self, agentId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check agent.

        Args:
            agentId: agent id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/agents/{agentId}", "HEAD", queryParams=queries, **kwargs)

    def deleteAgents(
        self, agentIds: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete agents.

        Args:
            agentIds: agent ids
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        payload = RequestPayload.buildJson(body={"agent_ids": agentIds})
        return self.makeRequest(f"{self.baseUri}/agents", "DELETE", queryParams=queries, body=payload, **kwargs)

    def deleteAgent(
        self, agentId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete agent.

        Args:
            agentId: agent id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/agents/{agentId}", "DELETE", queryParams=queries, **kwargs)

    def createAnalytic(
        self, accountId: str, analytic: Analytic, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new analytic.

        Args:
            accountId: account id
            analytic: analytic structure

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created group name and id will be returned.
        """
        payload = RequestPayload.buildJson(body=analytic.asDict())
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/analytics", "POST", body=payload, **kwargs)

    def getAnalytics(
        self,
        analyticIds: Optional[List[str]] = None,
        analyticIdGte: Optional[str] = None,
        analyticIdLt: Optional[str] = None,
        analyticNames: Optional[List[str]] = None,
        createTimeGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        accountId: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get analytics by filters.

        Args:
            accountId: account id
            analyticNames: analytic names
            analyticIds: analytic ids
            order: order
            analyticIdGte: analytic id upper included boundary
            analyticIdLt: analytic id lower excluded boundary
            createTimeGte: agent create time upper included boundary
            createTimeLt: agent create time lower excluded boundary
            page: page number
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with groups will be returned.
        """
        queries = {
            "account_id": accountId,
            "analytic_ids": analyticIds,
            "names": analyticNames,
            "analytic_id__gte": analyticIdGte,
            "analytic_id__lt": analyticIdLt,
            "create_time__gte": createTimeGte,
            "create_time__lt": createTimeLt,
            "order": order,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/analytics", "GET", queryParams=queries, **kwargs)

    def getAnalytic(
        self, analyticId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get analytic.

        Args:
            analyticId: analytic id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/analytics/{analyticId}", "GET", queryParams=queries, **kwargs)

    def getAnalyticDocumentation(
        self, analyticId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get analytic's documentation.

        Args:
            analyticId: analytic id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/analytics/{analyticId}/docs", "GET", queryParams=queries, **kwargs)

    def checkAnalytic(
        self, analyticId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check analytic.

        Args:
            analyticId: analytic id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with group will be returned.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/analytics/{analyticId}", "HEAD", queryParams=queries, **kwargs)

    def replaceAnalytic(
        self, analyticId: str, analytic: Analytic, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Replace analytic.

        Args:
            analyticId: analytic id
            analytic: analytic structure
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        payload = RequestPayload.buildJson(body=analytic.asDict())
        payload.applyAccountIdHeader(accountId)
        return self.makeRequest(f"{self.baseUri}/analytics/{analyticId}", "PUT", body=payload, **kwargs)

    def deleteAnalytics(
        self, analyticIds: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete analytics.

        Args:
            analyticIds: analytic ids
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        payload = RequestPayload.buildJson(body={"analytic_ids": analyticIds})
        return self.makeRequest(f"{self.baseUri}/analytics", "DELETE", queryParams=queries, body=payload, **kwargs)

    def deleteAnalytic(
        self, analyticId: str, accountId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete analytic.

        Args:
            analyticId: analytic id
            accountId: account_id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"account_id": accountId}
        return self.makeRequest(f"{self.baseUri}/analytics/{analyticId}", "DELETE", queryParams=queries, **kwargs)

    def getStreamsLogs(
        self,
        accountId: Optional[str] = None,
        streamIds: Optional[List[str]] = None,
        statuses: Optional[List[int]] = None,
        targets: Optional[List[str]] = None,
        logTimeLt: Optional[str] = None,
        logTimeGte: Optional[str] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get streams logs

        Args:
            accountId: account_id
            streamIds: stream ids
            statuses: streams statuses
            targets: log targets
            logTimeLt: log time lower excluded boundary
            logTimeGte: log time upper included boundary
            page: page
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {
            "account_id": accountId,
            "stream_ids": streamIds,
            "statuses": statuses,
            "targets": targets,
            "log_time__lt": logTimeLt,
            "log_time__gte": logTimeGte,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/streams/logs", "GET", queryParams=queries, **kwargs)

    def deleteStreamsLogs(
        self, logTimeLt: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete streams logs

        Args:
            logTimeLt: log time lower excluded boundary

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queries = {"log_time__lt": logTimeLt}
        return self.makeRequest(f"{self.baseUri}/streams/logs", "DELETE", queryParams=queries, **kwargs)

    def connect(
        self,
        streamId: str,
        accountId: str,
        asyncRequest: Optional[bool] = None,
        messageReceiveTimeout: Optional[Real] = None,
    ) -> Union[aiohttp.client._WSRequestContextManager, contextmanager]:  # pylint: disable=W0212
        """
        Create ws-connection to luna-video-manager

        Args:
            streamId: stream id
            accountId: account id to connect to

            asyncRequest: execution in asynchronous mode, disabled by default
            messageReceiveTimeout: message receive timeout (time between start waiting message and message appearance)
        Returns:
            context manager with connection to luna-video-manager
        """
        messageTimeout = messageReceiveTimeout or self.messageReceiveTimeout
        headers = self.getAccountIdAsDict(accountId)
        urlWithParams = "{}{}".format(self._baseWSUri, f"/streams/{streamId}/ws")

        if self.getAsyncMode(asyncRequest):
            return self.session.ws_connect(urlWithParams, headers=headers, receive_timeout=messageTimeout)

        @contextmanager
        def syncWebsocketContextManager():
            webSocketConnection = create_connection(url=urlWithParams, header=headers, timeout=messageTimeout)
            try:
                yield webSocketConnection
            finally:
                webSocketConnection.close()

        return syncWebsocketContextManager()

    def getRestream(
        self,
        streamId: str,
        restreamType: Literal["hls"] = "hls",
        restreamQuality: Optional[int] = None,
        accountId: Optional[str] = None,
        **kwargs,
    ):
        """
        Get restream

        Args:
            accountId: account id
            streamId: stream id
            restreamType: restream type
            restreamQuality: restream quality
        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
        """
        queryParams = {}
        if accountId:
            queryParams["account_id"] = accountId
        if restreamType:
            queryParams["type"] = restreamType
        if restreamQuality:
            queryParams["quality"] = restreamQuality
        return self.makeRequest(f"{self.baseUri}/streams/{streamId}/restream", "GET", queryParams=queryParams, **kwargs)
